/* indicator.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

namespace IndicatorWeather {

/**
 * ServiceClient interface
 *
 * It is the DBus interface to our service
 *
 **/
#if LOCALBUILD
[DBus(name = "net.launchpad.indicator.weather.test.service")]
#else
[DBus(name = "net.launchpad.indicator.weather.service")]
#endif
interface ServiceClient : GLib.Object {
    public signal void display_mode_changed(int display_mode);
    public signal void icon_name_changed(string icon_name);
    public signal void label_markup_changed(string label_markup);

    public signal void menuitem_property_changed(string menuitem_key,
        string property, GLib.Variant value);

    public abstract int get_display_mode() throws IOError;
    public abstract string get_icon_name() throws IOError;
    public abstract string get_label_markup() throws IOError;
    public abstract void set_location(string location) throws IOError;
}

/**
 * IndicatorClient class
 *
 * It is the indicator entry point, as a subclass of Indicator.Object
 **/
public class IndicatorClient : Indicator.Object {

    private Indicator.ServiceManager    _service_manager = null;
    private ServiceClient               _service_client = null;
    private GLib.List<void *>           _entries = null;
    private Indicator.ObjectEntry       _entry;
    private Gtk.Image                   _image = null;
    private Gtk.Label                   _label = null;
    private DbusmenuGtk.Menu            _menu = null;
    private DbusmenuGtk.Client          _menu_client = null;
    private bool                        _connected = false;

    /**
     * Basic construction.
     *
     * We cannot chain a default constructor to the parent class,
     * so we use a construct element.
     */
    construct {
        // Create an empty entry list
        _entries = new GLib.List<void *>();

        debug("IndicatorWeather::IndicatorClient");
        // Initialize a Indicator.ServiceManager object.
        // It will attempt to start the service side, if it is not running already.
        // If the service dies it attempts to restart it automatically.
        // We'll get notified for any connection change.
        _service_manager = new Indicator.ServiceManager.version(
            DBUS_SERVICE_NAME, DBUS_SERVICE_VERSION);
        _service_manager.connection_change.connect(on_connection_change);
    }

    /**
     * Service Manager connection-change event.
     *
     * Depending on the connection status,
     * we load or unload our indicator entry.
     */
    public void on_connection_change(bool connected) {
        _connected = connected;
        debug("IndicatorClient: indicator service manager connected: %s",
            connected.to_string());

        // Cleanup existing entry
        if (_image != null) {
            debug("IndicatorClient: destroying indicator image");
            _image.destroy();
        }
        if (_label != null) {
            debug("IndicatorClient: destroying indicator label");
            _label.destroy();
        }
        if (_menu != null) {
            debug("IndicatorClient: destroying indicator menu");
            _menu.destroy();
        }
        if (_entries.length() > 0) {
            _entries.remove(&_entry);
            debug("IndicatorClient: sending entry-removed signal");
            entry_removed(_entry);
        }
        _service_client = null;
        _image = null;
        _label = null;
        _menu = null;
        _menu_client = null;

        // We delay a bit the entry initialization to avoid dbus sync problems
        if (connected)
            Timeout.add_seconds(1, init, Priority.DEFAULT);
    }

    /**
     * Entry initialization.
     *
     * We setup the indicator entry and notify listeners
     */
    bool init() {
        if (!_connected)
            return false;

        try {
            // we get a dbus proxy interface to communicate with our service
            debug("IndicatorClient: getting service proxy");
            _service_client = Bus.get_proxy_sync(BusType.SESSION,
                DBUS_SERVICE_NAME2, DBUS_SERVICE_OBJ, DBusProxyFlags.NONE, null);
        } catch (Error e) {
            // this should not happen
            critical("IndicatorClient: init service error: %s", e.message);
            return false;
        }

        debug("IndicatorClient: creating dbusmenu client");
        _image = new Gtk.Image.from_stock(Gtk.Stock.MISSING_IMAGE, Gtk.IconSize.MENU);
        _label = new Gtk.Label("WI");
        // create the dbusmenu gtk client
        _menu = new DbusmenuGtk.Menu(DBUS_SERVICE_NAME, DBUS_MENU_OBJ);
        _menu.key_press_event.connect(on_menu_key_press_event);
        _menu.key_release_event.connect(on_menu_key_release_event);
        _menu.show();
        _menu_client = _menu.get_client();
        // register our custom menuitem widgets
        _menu_client.add_type_handler_full(MENUITEM_TYPE_GENERIC,
            GenericMenuItem.new_menuitem);
        _menu_client.add_type_handler_full(MENUITEM_TYPE_LOCATION,
            LocationMenuItem.new_menuitem);
        _menu_client.add_type_handler_full(MENUITEM_TYPE_CONDITION,
            ConditionMenuItem.new_menuitem);
        _menu_client.add_type_handler_full(MENUITEM_TYPE_TABLE,
            TableMenuItem.new_menuitem);
        _menu_client.add_type_handler_full(MENUITEM_TYPE_SEPARATOR,
            SeparatorMenuItem.new_menuitem);

        // setup image & label
        try {
            int display_mode = _service_client.get_display_mode();
            _image.set_visible(display_mode == DisplayMode.ICON ||
                                display_mode == DisplayMode.BOTH);
            _label.set_visible(display_mode == DisplayMode.LABEL ||
                                display_mode == DisplayMode.BOTH);
        } catch (Error e) {
            critical("IndicatorClient: get_display_mode service error: %s",
                e.message);
            _image.set_visible(true);
            _label.set_visible(false);
        }
        try {
            _image.set_from_icon_name(_service_client.get_icon_name(),
                                    Gtk.IconSize.MENU);
            _image.set_pixel_size (22);
        } catch (Error e) {
            critical("IndicatorClient: get_icon_name service error: %s",
                e.message);
            _image.set_from_icon_name(Gtk.Stock.MISSING_IMAGE, Gtk.IconSize.MENU);
            _image.set_pixel_size (22);
        }
        try {
            _label.set_markup(_service_client.get_label_markup());
        } catch (Error e) {
            critical("IndicatorClient: get_label_markup service error: %s",
                e.message);
            _label.set_markup("");
        }

        // connect to signals for changes
        debug("IndicatorClient: connecting to service signals");
        _service_client.display_mode_changed.connect(on_display_mode_changed);
        _service_client.icon_name_changed.connect(on_icon_name_changed);
        _service_client.label_markup_changed.connect(on_label_markup_changed);

        // create the entry
        // we have redefine it as a struct (instead of class) in indicator.vapi
        // otherwise it is impossible to initialize it
        debug("IndicatorClient: creating menu entry");
        _entry = Indicator.ObjectEntry();
        _entry.image = _image;
        _entry.label = _label;
        _entry.menu = _menu;
        _entries.prepend(&_entry);
        // notify listeners
        debug("IndicatorClient: sending entry-added signal");
        entry_added(_entry);
        return false;
    }

    /**
     * Indicator.Object.get_entries override.
     *
     * It will return an empty list in the begining.
     */
    public override unowned GLib.List get_entries() {
        return _entries;
    }

    /**
     * ServiceClient display-mode-changed event handler
     */
    public void on_display_mode_changed(int display_mode) {
        debug("IndicatorClient: display-mode changed: %d", display_mode);
        if (_image != null)
            _image.set_visible(display_mode == DisplayMode.ICON ||
                                display_mode == DisplayMode.BOTH);
        if (_label != null)
            _label.set_visible(display_mode == DisplayMode.LABEL ||
                                display_mode == DisplayMode.BOTH);
    }

    /**
     * ServiceClient icon-name-changed event handler
     */
    public void on_icon_name_changed(string icon_name) {
        debug("IndicatorClient: icon-name changed: %s", icon_name);
        if (_image != null) {
            _image.set_from_icon_name(icon_name, Gtk.IconSize.MENU);
            _image.set_pixel_size (22);
        }
    }

    /**
     * ServiceClient icon-name-changed event handler
     */
    public void on_label_markup_changed(string markup) {
        debug("IndicatorClient: label-markup changed: %s", markup);
        if (_label != null)
            _label.set_markup(markup);
    }
}

//namespace IndicatorWeather
}

/**
 * Indicator module get_version requirement
 */
public static string get_version() {
    return Indicator.VERSION;
}

/**
 * Indicator module get_type requirement
 */
public static GLib.Type get_type() {
    return typeof(IndicatorWeather.IndicatorClient);
}
