/* menu.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Padding on the right side of the menu
 */
public static const int MENU_PADDING = 2;

/**
 * Movement enum
 */
public enum Movement {
    NONE,
    ENTER,
    LEFT,
    RIGHT,
    UP,
    DOWN,
    PAGE_UP,
    PAGE_DOWN;

    /**
     * Returns a Movement instance from a key event
     */
    public static Movement from_key_event(Gdk.EventKey event) {
        // constants taken from /usr/include/gtk-2.0/gdk/gdkkeysyms.h
        switch(event.keyval) {
        case 0xff0d:  //GDK_KEY_Return
        case 0x020:   //GDK_KEY_space
        case 0xff8d:  //GDK_KEY_KP_Enter
        case 0xff80:  //GDK_KEY_KP_Space
            return Movement.ENTER;

        case 0xff51: //GDK_KEY_Left
        case 0xff96: //GDK_KEY_KP_Left
            return Movement.LEFT;

        case 0xff52: //GDK_KEY_Up
        case 0xff97: //GDK_KEY_KP_Up
            return Movement.UP;

        case 0xff53: //GDK_KEY_Right
        case 0xff98: //GDK_KEY_KP_Right
            return Movement.RIGHT;

        case 0xff54: //GDK_KEY_Down
        case 0xff99: //GDK_KEY_KP_Down
            return Movement.DOWN;

        case 0xff55: //GDK_KEY_Page_Up
        case 0xff9a: //GDK_KEY_KP_Page_Up
        case 0xff50: //GDK_KEY_Home
        case 0xff95: //GDK_KEY_KP_Home
            return Movement.PAGE_UP;

        case 0xff56: //GDK_KEY_Page_Down
        case 0xff9b: //GDK_KEY_KP_Page_Down
        case 0xff57: //GDK_KEY_End
        case 0xff9c: //GDK_KEY_KP_End
            return Movement.PAGE_DOWN;

        default:
            return Movement.NONE;
        }
    }
}

/**
 * Callback for key pressed on the menu
 *
 * We override the default behaviour because:
 * - we don't want the non-selectable menuitems to be navigated
 * - we don't want always to dismiss menu after menuitem activation
 */
public static bool on_menu_key_press_event(Widget widget,
                                           Gdk.EventKey event) {
    debug("IndicatorClient: menu key-press: %x, %s", event.keyval,
        Gdk.keyval_name(event.keyval));
    // this determines if we eat the event
    bool digested = false;
    // get movement
    Movement movement = Movement.from_key_event(event);
    // get menu and active item
    Menu menu = (Menu)widget;
    MenuItem menuitem = (MenuItem)menu.active_menu_item;

    // respond to movement
    if (movement == Movement.ENTER) {
        // delegate event to BaseMenuItem
        if (menuitem is BaseMenuItem)
            digested = ((BaseMenuItem)menuitem).handle_selection(true);

    } else if (movement == Movement.LEFT || movement == Movement.RIGHT) {
        // delegate event to SelectionMenuItem
        if (menuitem is SelectionMenuItem)
            digested = ((SelectionMenuItem)menuitem).handle_movement(
                false, movement);

    } else if (movement == Movement.UP || movement == Movement.DOWN ||
               movement == Movement.PAGE_UP || movement == Movement.PAGE_DOWN) {
        // eat the event
        digested = true;
        // find the next proper selectable item
        // and select it
        unowned List<MenuItem> children = menu.children;
        if (children.length() > 0) {
            unowned List<MenuItem> curr_child = null;
            unowned List<MenuItem> start_child = null;
            // find current child
            if (menuitem != null)
                curr_child = children.find(menuitem);
            if (movement == Movement.UP || movement == Movement.DOWN) {
                // start with the current child
                start_child = curr_child;
                if (start_child == null) {
                    // or if it's not set yet:
                    // if we go up, start with the first
                    // if we go down, start with the last
                    start_child = movement == Movement.UP ?
                        children.first() : children.last();
                }
            } else {
                // if we go up, start with the last
                // if we go down, start with the first
                start_child = movement == Movement.PAGE_UP ?
                    children.last() : children.first();
            }
            unowned List<MenuItem> child = start_child;
            bool exit = false;
            do {
                // handle movement inside a LocationMenuItem
                if (child.data is LocationMenuItem) {
                    if (((LocationMenuItem)child.data).handle_movement(
                            false, movement))
                        break;
                }
                // proceed with next menuitem
                if (movement == Movement.UP || movement == Movement.PAGE_DOWN) {
                    // pick the previous child
                    // in overflow pick the last
                    child = child.prev;
                    if (child == null)
                        child = children.last();
                } else {
                    // pick the next child
                    // in overflow pick the first
                    child = child.next;
                    if (child == null)
                        child = children.first();
                }
                // if we have a different child
                exit = (child == null ||
                    child == start_child ||
                    child == curr_child);
                if (!exit) {
                    // get the menuitem
                    MenuItem mi = child.data;
                    // if it can be selected
                    if (mi.get_visible() && mi.is_sensitive() &&
                            !(mi is SeparatorMenuItem)) {
                        // ok we have a found a possible target
                        bool found = true;
                        // if it is a BaseMenuItem
                        // check the selectable flag
                        if (mi is BaseMenuItem)
                            found = ((BaseMenuItem)mi).is_selectable();
                        if (found) {
                            debug("IndicatorClient: selecting menuitem: %s",
                                mi.get_name());
                            // select found item
                            menu.select_item(mi);
                            // if it is a SelectionMenuItem
                            // set the selected widget
                            if (mi is SelectionMenuItem)
                                ((SelectionMenuItem)mi).handle_movement(
                                    true, movement);
                            // exit loop
                            break;
                        }
                    }
                }
            } while (!exit);
        }
    }
    return digested;
}

/**
 * Callback for key release on the menu
 *
 * Here we mainly eat the event based on the movement
 */
public static bool on_menu_key_release_event(Widget widget,
                                             Gdk.EventKey event) {
    debug("IndicatorClient: menu key-release: %x, %s", event.keyval,
        Gdk.keyval_name(event.keyval));
    // this determines if we eat the event
    bool digested = false;
    // get movement
    Movement movement = Movement.from_key_event(event);
    // get menu and active item
    Menu menu = (Menu)widget;
    MenuItem menuitem = (MenuItem)menu.active_menu_item;
    // respond to movement
    if (movement == Movement.ENTER) {
        // delegate event to BaseMenuItem
        if (menuitem is BaseMenuItem)
            digested = ((BaseMenuItem)menuitem).handle_selection(false);

    } else if (movement == Movement.LEFT ||
               movement == Movement.RIGHT) {
        // delegate event to SelectionMenuItem
        if (menuitem is SelectionMenuItem)
            digested = ((SelectionMenuItem)menuitem).is_selectable();

    } else if (movement == Movement.UP ||
               movement == Movement.DOWN ||
               movement == Movement.PAGE_UP ||
               movement == Movement.PAGE_DOWN) {
        // eat the event
        digested = true;
    }
    return digested;
}

//namespace IndicatorWeather
}
