/* menuitem_base.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Base menuitem
 *
 * Subclass of Gtk.MenuItem
 *
 * An abstract widget that used as base for all our custom widgets.
 * It handles all common operation & properties.
 *
 * Custom properties:
 * MENUITEM_PROP_KEY: The unique key identifier
 * MENUITEM_PROP_SELECTABLE: controls whether the menuitem can be navigated,
 *                           acted on and displayed as selected
 * MENUITEM_PROP_DISMISS_MENU: controls whether the menu should be dismissed,
 *                             after menuitem activation
 **/
abstract class BaseMenuItem : Gtk.MenuItem {

    /**
     * Reference to the Dbus menuitem we listen to.
     * Defined as weak to avoid cycled refs.
     */
    protected weak Dbusmenu.Menuitem _dbusitem = null;

    /**
     * The menuitem key
     */
    protected string _key;

    /**
     * Is the widget selectable?
     */
    protected bool _selectable = true;

    /**
     * Dismiss menu on activate?
     */
    protected bool _dismiss_menu = true;

    /**
     * Reference to the service interface.
     */
    protected ServiceClient _service_client = null;

    /**
     * Widget direction
     */
    protected TextDirection _dir;

    /**
     * Constructor
     */
    public BaseMenuItem(Dbusmenu.Menuitem item) {

        _dbusitem = item;
        _key = _dbusitem.property_get(MENUITEM_PROP_KEY);

        name = "%s[%s]".printf(this.get_type().name().
            replace("IndicatorWeather", "").replace("MenuItem", ""), _key);
        debug("%s created", name);

        try {
            _service_client = Bus.get_proxy_sync(BusType.SESSION,
                DBUS_SERVICE_NAME2, DBUS_SERVICE_OBJ,
                DBusProxyFlags.NONE, null);
        } catch (Error e) {
            critical("%s init: service error: %s",
                name, e.message);
        }

        _dir = get_default_direction();
    }

    public override void destroy() {
        debug("%s destroy", name);
        base.destroy();
    }

    public override void realize() {
        debug("%s realized", name);
        base.realize();
    }

    protected void connect_events() {
        if (_dbusitem.property_exist(Dbusmenu.MENUITEM_PROP_VISIBLE))
            set_visible(_dbusitem.property_get_bool(
                Dbusmenu.MENUITEM_PROP_VISIBLE));
        if (_dbusitem.property_exist(Dbusmenu.MENUITEM_PROP_ENABLED))
            set_sensitive(_dbusitem.property_get_bool(
                Dbusmenu.MENUITEM_PROP_ENABLED));
        if (_dbusitem.property_exist(MENUITEM_PROP_SELECTABLE))
            _set_selectable(_dbusitem.property_get_bool(
                MENUITEM_PROP_SELECTABLE));
        if (_dbusitem.property_exist(MENUITEM_PROP_DISMISS_MENU))
            _set_dismiss_menu(_dbusitem.property_get_bool(
                MENUITEM_PROP_DISMISS_MENU));

        // setup the property_changed listener
        if (USE_SERVICE_CLIENT_PROP_CHANGE && _service_client != null)
            _service_client.menuitem_property_changed.connect(
                on_menuitem_property_changed);
        else
            _dbusitem.property_changed.connect(on_property_changed);

        // setup the event listener
        _dbusitem.event.connect(on_dbusitem_event);
    }

    public bool on_dbusitem_event (string name, GLib.Variant value,
                                    uint timestamp) {
        if (name == Dbusmenu.MENUITEM_EVENT_ACTIVATED && !_selectable)
            return true;
        return false;
    }

    public void on_menuitem_property_changed (string menuitem_key,
            string property, GLib.Variant value) {
        if (menuitem_key == _key)
            on_property_changed(property, value);
    }

    public virtual void on_property_changed (string property,
                                             GLib.Variant value) {

        if (property == MENUITEM_PROP_SELECTABLE) {
            _set_selectable(value.get_boolean());

        } else if (property == MENUITEM_PROP_DISMISS_MENU) {
            _set_dismiss_menu(value.get_boolean());

        } else if (property == Dbusmenu.MENUITEM_PROP_VISIBLE) {
            set_visible(value.get_boolean());
            if (!get_visible() && get_sensitive())
                set_state(StateType.NORMAL);

        } else if (property == Dbusmenu.MENUITEM_PROP_ENABLED) {
            set_sensitive(value.get_boolean());

        } else {
            debug("%s: unhandled property changed: %s", name, property);
        }
    }

    protected void _set_selectable(bool selectable) {
        debug("%s: selectable: %s", name, selectable.to_string());
        _selectable = selectable;
    }
    protected void _set_dismiss_menu(bool dismiss_menu) {
        debug("%s: dismiss menu: %s", name, dismiss_menu.to_string());
        _dismiss_menu = dismiss_menu;
    }

    public override void select() {
        // eat the event to appear inactive if we are not selectable
        if (_selectable)
            base.select();
    }
    public override void deselect() {
        // eat the event to appear inactive if we are not selectable
        if (_selectable)
            base.deselect();
    }

    /**
     * Handles the activation events (key/mouse)
     * If act parameter is true (key_press/button_release)
     * invokes the activation
     *
     * Returns true if the event is handled
     */
    public virtual bool handle_selection(bool act) {
        // eat the event to not dismiss menu
        // if we are not selectable or
        // if we don't want to dismiss the menu
        if (_selectable) {
            if (act && !_dismiss_menu)
                activate();
            return !_dismiss_menu;
        } else {
            return true;
        }
    }

    public override bool button_press_event(Gdk.EventButton event) {
        return handle_selection(false);
    }

    public override bool button_release_event(Gdk.EventButton event) {
        return handle_selection(true);
    }

    public bool is_selectable () {
        return _selectable;
    }
}

//namespace IndicatorWeather
}

