/* menuitem_condition.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Condition menuitem
 *
 * Subclass of BaseMenuItem
 *
 * It displays the current conditions using 3 labels and an icon.
 *
 * The element contents are controled by these properties:
 * MENUITEM_PROP_LABEL  : text for main label widget ("Now:"/"Today:")
 * MENUITEM_PROP_LABEL2 : text for secondary label widget (the condition label)
 * MENUITEM_PROP_DETAIL : text for detail label widget (the temperature)
 * MENUITEM_PROP_ICON   : icon_name for image widget (the condition icon)
 * MENUITEM_PROP_ICON_SIZE : pixel size for image widget
 *
 **/
class ConditionMenuItem : BaseMenuItem {

    /**
     * The internal child
     */
    private Box _box_container;

    /**
     * The main label widget
     */
    private Label _lbl_main;

    /**
     * The secondary label widget
     */
    private Label _lbl_secondary;

    /**
     * The detail label widget
     */
    private Label _lbl_detail;

    /**
     * The label fields container
     */
    private Box _box_fields;

    /**
     * The label fields alignment container
     */
    private Alignment _ali_fields;

    /**
     * The condition image widget
     */
    private Image _img_icon;

    /**
     * true if valid icon is set
     */
    private bool _icon_is_set = false;

    /**
     * Icon pixel size
     */
    private int _icon_size = 48;

    /**
     * Space between rows
     */
    private int _row_margin = 6;
    /**
     * Space between columns
     */
    private int _col_margin = 5;

    /**
     * Layout to use
     */
    private ConditionItemLayout _layout = ConditionItemLayout.COMPACT2;

    /**
     * Hidden constructor
     */
    private ConditionMenuItem(Dbusmenu.Menuitem item) {
        base(item);
        _selectable = false;
        _dismiss_menu = false;

        _box_container = new HBox(false, _col_margin);
        _box_fields = new VBox(false, _row_margin);
        HBox _box_icon = new HBox(false, 0);

        _lbl_main = new Label(null);
        _lbl_secondary = new Label(null);
        _lbl_detail = new Label(null);
        _img_icon = new Image();

        const int LABEL_PADDING = 10;
        if (_dbusitem.property_exist(MENUITEM_PROP_LAYOUT))
            _layout = (ConditionItemLayout)_dbusitem.property_get_int(
                MENUITEM_PROP_LAYOUT);

        if (_layout == ConditionItemLayout.PLAIN) {
            if (_dir == TextDirection.RTL) {
                _lbl_main.set_alignment(1, 0.5f);
                _lbl_secondary.set_alignment(1, 0.5f);
                _lbl_detail.set_alignment(1, 0.5f);
            } else {
                _lbl_main.set_alignment(0, 0.5f);
                _lbl_secondary.set_alignment(0, 0.5f);
                _lbl_detail.set_alignment(0, 0.5f);
            }
            _box_fields.pack_start(_lbl_main);
            _box_fields.pack_start(_lbl_secondary);
            _box_fields.pack_start(_lbl_detail);
            _box_icon.pack_start(_img_icon, false);
            _lbl_secondary.set_padding(LABEL_PADDING, 0);
            _lbl_detail.set_padding(LABEL_PADDING, 0);

        } if (_layout == ConditionItemLayout.COMPACT) {
            HBox _box_title = new HBox(false, LABEL_PADDING);
            if (_dir == TextDirection.RTL) {
                _box_title.pack_start(_lbl_detail, false);
                _box_title.pack_start(_lbl_main);
                _lbl_main.set_alignment(1, 0.5f);
                _lbl_secondary.set_alignment(1, 0.5f);
                _lbl_detail.set_alignment(1, 0.5f);
            } else {
                _box_title.pack_start(_lbl_main, false);
                _box_title.pack_start(_lbl_detail);
                _lbl_main.set_alignment(0, 0.5f);
                _lbl_secondary.set_alignment(0, 0.5f);
                _lbl_detail.set_alignment(0, 0.5f);
            }
            _box_fields.pack_start(_box_title);
            _box_fields.pack_start(_lbl_secondary);
            _box_icon.pack_start(_img_icon, false);
            _lbl_secondary.set_padding(LABEL_PADDING, 0);

        } else if (_layout == ConditionItemLayout.COMPACT2) {
            if (_dir == TextDirection.RTL) {
                _lbl_main.set_alignment(1, 0.5f);
                _lbl_secondary.set_alignment(1, 0.5f);
            } else {
                _lbl_main.set_alignment(0, 0.5f);
                _lbl_secondary.set_alignment(0, 0.5f);
            }
            _lbl_detail.set_alignment(0, 0);
            _box_fields.pack_start(_lbl_main);
            _box_fields.pack_start(_lbl_secondary);
            if (_dir == TextDirection.RTL) {
                _box_icon.pack_start(_img_icon, false);
                _box_icon.pack_start(_lbl_detail, false);
            } else {
                _box_icon.pack_start(_lbl_detail, false);
                _box_icon.pack_start(_img_icon, false);
            }
            _lbl_secondary.set_padding(LABEL_PADDING, 0);
        }

        _ali_fields = new Alignment(0, 0, 0, 0);
        _ali_fields.set_padding(0, 0, 0, 0);
        _ali_fields.add(_box_fields);

        // arrange according the language default text direction
        if (_dir == TextDirection.RTL) {
            _ali_fields.set(1, 0, 1, 0);
            _box_container.pack_start(_box_icon, false);
            _box_container.pack_start(_ali_fields, true);
//            _box_icon.set_alignment(0, 0);
        } else {
            _ali_fields.set(0, 0, 1, 0);
            _box_container.pack_start(_ali_fields, true);
            _box_container.pack_start(_box_icon, false);
//            _box_icon.set_alignment(1, 0);
        }

        _box_container.show_all();
        this.add(_box_container);
        _update();
    }

    /**
     * Standard construction method called from the dbusmenu client.
     *
     * Creates an instance and registers it.
     */
    public static bool new_menuitem (Dbusmenu.Menuitem newitem,
        Dbusmenu.Menuitem parent, Dbusmenu.Client client) {

        ((DbusmenuGtk.Client)client).newitem_base(newitem,
            new ConditionMenuItem(newitem), parent);
        return true;
    }

    private void _update() {
        // update the widgets
        if (_dbusitem.property_exist(MENUITEM_PROP_LABEL))
            _set_main_label(_dbusitem.property_get(
                MENUITEM_PROP_LABEL));
        if (_dbusitem.property_exist(MENUITEM_PROP_LABEL2))
            _set_secondary_label(_dbusitem.property_get(
                MENUITEM_PROP_LABEL2));
        if (_dbusitem.property_exist(MENUITEM_PROP_DETAIL))
            _set_detail_label(_dbusitem.property_get(
                MENUITEM_PROP_DETAIL));
        if (_dbusitem.property_exist(MENUITEM_PROP_ICON_SIZE))
            _set_icon_size(_dbusitem.property_get_int(
                MENUITEM_PROP_ICON_SIZE));
        if (_dbusitem.property_exist(MENUITEM_PROP_ICON))
            _set_icon_name(_dbusitem.property_get(
                MENUITEM_PROP_ICON));
        if (_dbusitem.property_exist(MENUITEM_PROP_COL_MARGIN))
            _set_col_margin(_dbusitem.property_get_int(
                MENUITEM_PROP_COL_MARGIN));
        if (_dbusitem.property_exist(MENUITEM_PROP_ROW_MARGIN))
            _set_row_margin(_dbusitem.property_get_int(
                MENUITEM_PROP_ROW_MARGIN));

        connect_events();
    }

    public override void on_property_changed(string property,
                                             GLib.Variant value) {
        if (property == MENUITEM_PROP_LABEL) {
            _set_main_label(value.get_string());

        } else if (property == MENUITEM_PROP_LABEL2) {
            _set_secondary_label(value.get_string());

        } else if (property == MENUITEM_PROP_DETAIL) {
            _set_detail_label(value.get_string());

        } else if (property == MENUITEM_PROP_ICON) {
            _set_icon_name(value.get_string());

        } else if (property == MENUITEM_PROP_ICON_SIZE) {
            _set_icon_size(value.get_int32());

        } else if (property == MENUITEM_PROP_COL_MARGIN) {
            _set_col_margin(value.get_int32());

        } else if (property == MENUITEM_PROP_ROW_MARGIN) {
            _set_row_margin(value.get_int32());

        } else {
            base.on_property_changed(property, value);
        }
    }

    private void _set_main_label(string? markup) {
        debug("%s: main label: %s", name, markup);
        _lbl_main.set_markup(markup);
    }
    private void _set_secondary_label(string? markup) {
        debug("%s: secondary label: %s", name, markup);
        _lbl_secondary.set_markup(markup);
    }
    private void _set_detail_label(string? markup) {
        debug("%s: detail label: %s", name, markup);
        _lbl_detail.set_markup(markup);
    }
    private void _set_icon_name(string? icon_name) {
        debug("%s: icon_name: %s", name, icon_name);
        _icon_is_set = false;
        if (icon_name != null && icon_name != MISSING_VALUE) {
            IconTheme theme = IconTheme.get_default();
            if (theme.has_icon(icon_name)) {
                _img_icon.set_from_icon_name(icon_name, IconSize.MENU);
                _img_icon.set_pixel_size(_icon_size);
                _icon_is_set = true;
                _img_icon.show();
            } else {
                debug("%s: unknown icon name: %s", name, icon_name);
                _img_icon.hide();
            }

        } else {
            _img_icon.hide();
        }
    }
    private void _set_icon_size(int icon_size) {
        debug("%s: icon_size: %d", name, icon_size);
        _icon_size = icon_size;
        if (_icon_is_set)
            _img_icon.set_pixel_size(_icon_size);
    }
    private void _set_col_margin(int col_margin) {
        debug("%s: col_margin: %d", name, col_margin);
        _col_margin = col_margin;
        _box_container.set_spacing(_col_margin);
    }
    private void _set_row_margin(int row_margin) {
        debug("%s: row_margin: %d", name, row_margin);
        _row_margin = row_margin;
        _box_fields.set_spacing(_row_margin);
        //_ali_fields.set_padding(0, _row_margin, 0, 0);
    }
}

//namespace IndicatorWeather
}

