/* menuitem_generic.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Generic menuitem
 *
 * Subclass of BaseMenuItem
 *
 * A generic widget that displays a main label, an icon and a detail label
 * horizontally.
 *
 * The element contents are controled by these (string) properties:
 * MENUITEM_PROP_LABEL  : text for main label widget
 * MENUITEM_PROP_DETAIL : text for detail label widget
 * MENUITEM_PROP_ICON   : icon name for image widget
 **/
class GenericMenuItem : BaseMenuItem {

    /**
     * The widget container
     */
    private Box _box_container;

    /**
     * The main label widget
     */
    private Label _lbl_main;

    /**
     * The detail label widget
     */
    private Label _lbl_detail;

    /**
     * The image widget
     */
    private Image _img_icon;

    /**
     * true if valid icon is set
     */
    private bool _icon_is_set = false;

    /**
     * Space between columns
     */
    private int _col_margin = 5;

    /**
     * Hidden constructor
     */
    private GenericMenuItem(Dbusmenu.Menuitem item) {
        base(item);
        _selectable = true;
        _dismiss_menu = true;

        _box_container = new HBox(false, _col_margin);

        _lbl_main = new Label(null);
        _lbl_detail = new Label(null);
        _img_icon = new Image();

        // arrange according the language default text direction
        if (_dir == TextDirection.RTL) {
            _box_container.pack_start(_lbl_detail, false);
            _box_container.pack_start(_img_icon, false);
            _box_container.pack_start(_lbl_main, true);
            _lbl_main.set_alignment (1, 0.5f);
            _lbl_detail.set_alignment (1, 0.5f);
        } else {
            _box_container.pack_start(_lbl_main, true);
            _box_container.pack_start(_img_icon, false);
            _box_container.pack_start(_lbl_detail, false);
            _lbl_main.set_alignment (0, 0.5f);
            _lbl_detail.set_alignment (0, 0.5f);
        }

        // we want a padding on the opposite side of the item
        HBox wrapper = new HBox(false, 0);
        Label margin = new Label(null);
        margin.set_padding(MENU_PADDING,0);
        if (_dir == TextDirection.RTL) {
            wrapper.pack_start(margin, false);
            wrapper.pack_start(_box_container);
        } else {
            wrapper.pack_start(_box_container);
            wrapper.pack_start(margin, false);
        }
        wrapper.show_all();
        this.add(wrapper);
        _update();
    }

    /**
     * Standard construction method called from the dbusmenu client.
     *
     * Creates an instance and registers it.
     */
    public static bool new_menuitem (Dbusmenu.Menuitem newitem,
        Dbusmenu.Menuitem parent, Dbusmenu.Client client) {

        ((DbusmenuGtk.Client)client).newitem_base(newitem,
            new GenericMenuItem(newitem), parent);
        return true;
    }

    private void _update() {
        // update the widgets
        if (_dbusitem.property_exist(MENUITEM_PROP_LABEL))
            _set_main_label(_dbusitem.property_get(
                MENUITEM_PROP_LABEL));
        if (_dbusitem.property_exist(MENUITEM_PROP_DETAIL))
            _set_detail_label(_dbusitem.property_get(
                MENUITEM_PROP_DETAIL));
        if (_dbusitem.property_exist(MENUITEM_PROP_ICON))
            _set_icon_name(_dbusitem.property_get(
                MENUITEM_PROP_ICON));
        if (_dbusitem.property_exist(MENUITEM_PROP_COL_MARGIN))
            _set_col_margin(_dbusitem.property_get_int(
                MENUITEM_PROP_COL_MARGIN));

        connect_events();
    }

    public override void on_property_changed(string property,
                                             GLib.Variant value) {
        if (property == MENUITEM_PROP_LABEL) {
            _set_main_label(value.get_string());

        } else if (property == MENUITEM_PROP_DETAIL) {
            _set_detail_label(value.get_string());

        } else if (property == MENUITEM_PROP_ICON) {
            _set_icon_name(value.get_string());

        } else if (property == MENUITEM_PROP_COL_MARGIN) {
            _set_col_margin(value.get_int32());

        } else {
            base.on_property_changed(property, value);
        }
    }

    private void _set_main_label(string? markup) {
        debug("%s: main label: %s", name, markup);
        _lbl_main.set_markup(markup);
    }
    private void _set_detail_label(string? markup) {
        debug("%s: detail label: %s", name, markup);
        _lbl_detail.set_markup(markup);
    }
    private void _set_icon_name(string? icon_name) {
        debug("%s: icon_name: %s", name, icon_name);
        _icon_is_set = false;
        if (icon_name != null && icon_name != MISSING_VALUE) {
            IconTheme theme = IconTheme.get_default();
            if (theme.has_icon(icon_name)) {
                _img_icon.set_from_icon_name(icon_name, IconSize.MENU);
                _icon_is_set = true;
                _img_icon.show();
            } else {
                debug("%s: unknown icon name: %s", name, icon_name);
                _img_icon.hide();
            }

        } else {
            _img_icon.hide();
        }
    }
    private void _set_col_margin(int col_margin) {
        debug("%s: col_margin: %d", name, col_margin);
        _col_margin = col_margin;
        _box_container.set_spacing(_col_margin);
    }

}

//namespace IndicatorWeather
}

