/* menuitem_location.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Location menuitem
 *
 * Subclass of SelectionMenuItem
 *
 * A carousel-like widget used to represent a selectable location.
 * It displays a main label, and two arrows in the sides.
 *
 * The element contents are controled by these (string) properties:
 * MENUITEM_PROP_LABEL  : text for main label widget
 * MENUITEM_PROP_ITEMS  : a string array with the location codes
 * MENUITEM_PROP_INDEX  : the array index of currently selected location
 *
 * When it is selected by the user,
 * it calls the service interface method "set_location"
 *
 **/
class LocationMenuItem : SelectionMenuItem {

    /**
     * The widget container
     */
    private Box _box_container;

    /**
     * The main label widget
     */
    private Label _lbl_main;

    /**
     * The left arrow widget
     */
    private Arrow _arr_left;
    /**
     * The right arrow widget
     */
    private Arrow _arr_right;

    /**
     * The carousel item array
     * Each element is a location code
     */
    private string[] _items;

    /**
     * The current item index
     */
    private int _index = 0;

    /**
     * Hidden constructor
     */
    private LocationMenuItem(Dbusmenu.Menuitem item) {
        base(item);
        _selectable = false;
        _dismiss_menu = false;

        _items = new string[] {};
        _index = 0;

        _box_container = new HBox(false, 0);

        _lbl_main = new Label(null);
        _arr_left = new Arrow(ArrowType.LEFT, ShadowType.OUT);
        _arr_right = new Arrow(ArrowType.RIGHT, ShadowType.OUT);
        _arr_left.set_name("left arrow");
        _arr_right.set_name("right arrow");
        _active_widgets.append(_arr_left);
        _active_widgets.append(_arr_right);

        _box_container.pack_start(_arr_left, false);
        _box_container.pack_start(_lbl_main, true);
        _box_container.pack_start(_arr_right, false);

        Alignment wrapper = new Alignment(0.5f, 0.5f, 0.5f, 1f);
        wrapper.add(_box_container);
        wrapper.show_all();
        this.add(wrapper);

        _update();
    }

    /**
     * Standard construction method called from the dbusmenu client.
     *
     * Creates an instance and registers it.
     */
    public static bool new_menuitem (Dbusmenu.Menuitem newitem,
        Dbusmenu.Menuitem parent, Dbusmenu.Client client) {

        ((DbusmenuGtk.Client)client).newitem_base(newitem,
            new LocationMenuItem(newitem), parent);
        return true;
    }

    public override void realize() {
        base.realize();
        // change the arrow selected color
        Gdk.Color clr = get_style().bg[StateType.SELECTED];
        _arr_left.modify_fg(StateType.SELECTED, clr);
        _arr_right.modify_fg(StateType.SELECTED, clr);
    }

    private void _update() {
        // update the widgets
        if (_dbusitem.property_exist(MENUITEM_PROP_LABEL))
            _set_main_label(_dbusitem.property_get(
                MENUITEM_PROP_LABEL));
        if (_dbusitem.property_exist(MENUITEM_PROP_ITEMS)) {
            Variant value = _dbusitem.property_get_variant(MENUITEM_PROP_ITEMS);
            _set_items(value.get_strv());
        }
        if (_dbusitem.property_exist(MENUITEM_PROP_INDEX))
            _set_index(_dbusitem.property_get_int(
                MENUITEM_PROP_INDEX));

        connect_events();
    }

    public override void on_property_changed (string property,
                                              GLib.Variant value) {
        if (property == MENUITEM_PROP_LABEL) {
            _set_main_label(value.get_string());

        } else if (property == MENUITEM_PROP_ITEMS) {
            _set_items(value.get_strv());

        } else if (property == MENUITEM_PROP_INDEX) {
            _set_index(value.get_int32());

        } else {
            base.on_property_changed(property, value);
        }
    }

    private void _set_main_label(string? markup) {
        debug("%s: main label: %s", name, markup);
        _lbl_main.set_markup(markup);
    }
    private void _set_items(string?[] items) {
        _items = items;
        debug("%s: set %d items", name, _items.length);
        for (int i = 0; i < items.length; i++)
            debug("%s: item %d: %s", name, i, _items[i]);
        if (_items.length < 2) {
            _arr_left.hide();
            _arr_right.hide();
        } else {
            _arr_left.show();
            _arr_right.show();
        }
        if (_items.length > 0)
            _set_index(_index);
        _set_selectable(_items.length > 1);
    }
    private void _set_index(int index) {
        _index = index;
    }

    private void _set_new_index(int new_index) {
        if (new_index < 0)
            new_index = _items.length - 1;
        else if (new_index >= _items.length)
            new_index = 0;
        string new_code = _items[new_index];
        try {
            debug("%s: activated '%s': %d '%s'", name,
                _previously_selected_widget.get_name(),
                new_index, new_code);
            _service_client.set_location(new_code);
        } catch (Error e) {
            critical("%s set_location: service error: %s",
                name, e.message);
        }
    }

    public override bool handle_movement(bool landed, Movement movement) {
        bool result = base.handle_movement(landed, movement);
        if (result && (movement == Movement.LEFT || movement == Movement.RIGHT))
            _set_new_index(movement == Movement.LEFT ? _index - 1 : _index + 1);
        return result;
    }

    public override void on_selected_widget_action(Widget selected) {
        _set_new_index(selected == _arr_left ? _index - 1 : _index + 1);
    }
}

//namespace IndicatorWeather
}

