/* menuitem_selection.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Selection menuitem
 *
 * Subclass of BaseMenuItem
 *
 * An abstract widget that maintains a list of internal widgets,
 * which we should navigate on.
 *
 * Logic taken from Chromium's custom gtk menu
 **/

abstract class SelectionMenuItem : BaseMenuItem {

    /**
     * The active widget list
     */
    protected List<Widget> _active_widgets;
    /**
     * The previously selected widget
     */
    protected Widget _previously_selected_widget = null;
    /**
     * The currently selected widget
     */
    protected Widget _currently_selected_widget = null;

    /**
     * Constructor
     */
    public SelectionMenuItem(Dbusmenu.Menuitem item) {
        base(item);
        _active_widgets = new List<Widget>();
    }

    /**
     * Handles the movement inside the widget on response to menu key press
     * Returns true if the event is handled
     */
    public virtual bool handle_movement(bool landed, Movement movement) {

        uint cnt = _get_selection_widget_count();
        if (cnt > 0) {
            // just landed in the menuitem
            if (landed) {
                if (movement == Movement.UP ||
                    movement == Movement.PAGE_DOWN ||
                    movement == Movement.RIGHT) {
                    // select the last
                    _set_selected_widget_index(cnt - 1);
                } else {
                    // select the first
                    _set_selected_widget_index(0);
                }
                return true;
            }
            int index = _get_selected_widget_index();
            if (movement == Movement.UP || movement == Movement.LEFT) {
                if (index < 0) {
                    // if no selection exists select the last
                    _set_selected_widget_index(cnt - 1);
                    return true;

                } else if (index > 0) {
                    // if a selection exists and it's not the first
                    // select the previous
                    _set_selected_widget_index(index - 1);
                    return true;

                } else if (index == 0 && movement == Movement.LEFT) {
                    // if a selection is the first and we go left
                    // stay there
                    return true;
                }

            } else if (movement == Movement.DOWN || movement == Movement.RIGHT) {
                if (index < 0) {
                    // if no selection exists select the first
                    _set_selected_widget_index(0);
                    return true;

                } else if (index < cnt - 1) {
                    // if a selection exists and it's not the last
                    // select the next
                    _set_selected_widget_index(index + 1);
                    return true;

                } else if (index == cnt - 1 && movement == Movement.RIGHT) {
                    // if a selection is the last and we go right
                    // stay there
                    return true;
                }
            }
        }
        // in all other cases deselect
        deselect();
        return false;
    }

    /**
     * Handles the activation events (key/mouse)
     * If act parameter is true (key_press/button_release)
     * invokes the activation
     *
     * Returns true if the event is handled
     */
    public override bool handle_selection(bool act) {
        // eat the event to not dismiss menu
        // if we are not selectable or
        // if we don't want to dismiss the menu
        if (_selectable) {
            if (act && !_dismiss_menu) {
                deselect();
                activate();
            }
            return !_dismiss_menu;
        } else {
            return true;
        }
    }

    public abstract void on_selected_widget_action(Widget selected);

    public override void activate() {
        // We look at |_previously_selected_widget| because by the time we've
        // been activated, we've already gone through our deselect handler.
        if (_previously_selected_widget != null)
            _set_selected_widget(null);

        if (_previously_selected_widget != null)
            on_selected_widget_action(_previously_selected_widget);
    }


    public override void select() {
        // When we are selected, the only thing we do is clear information from
        // previous selections. Actual selection of a widget is done either in
        // the "mouse-motion-event" or is manually set from menu's
        // "key-pressed" callback.
        _previously_selected_widget = null;
        queue_draw();
    }

    public override void deselect() {
        // When we are deselected, we store the item that was currently selected
        // so that it can be acted on. Menu items are first deselected before
        // they are activated.
        _previously_selected_widget = _currently_selected_widget;
        if (_currently_selected_widget != null)
            _set_selected_widget(null);
        queue_draw();
    }

    public override bool motion_notify_event(Gdk.EventMotion event) {
        // Translate coords and find out if a widget is selected
        Widget new_selected_widget = null;
        unowned List<Widget> child = null;
        for (child = _active_widgets; child != null; child = child.next) {
            Widget widget = child.data;
            if (widget.get_visible()) {
                Allocation alloc;
                widget.get_allocation(out alloc);
                int offset_x, offset_y;
                widget.translate_coordinates(this, 0, 0,
                    out offset_x, out offset_y);
                if (event.x >= offset_x &&
                        event.x < (offset_x + alloc.width) &&
                        event.y >= offset_y &&
                        event.y < (offset_y + alloc.height)) {
                    new_selected_widget = widget;
                    break;
                }
            }
        }
        _set_selected_widget(new_selected_widget);
        return false;
    }

    protected void _set_selected_widget(Widget? selected) {
        if (selected != _currently_selected_widget) {
            if (_currently_selected_widget != null) {
                _currently_selected_widget.set_state(StateType.NORMAL);
            }
            _currently_selected_widget = selected;
            if (_currently_selected_widget != null) {
                _currently_selected_widget.set_state(StateType.SELECTED);
                debug("%s _set_selected_widget %s", name,
                    _currently_selected_widget.get_name());
            }
        }
    }

    protected void _set_selected_widget_index(uint index) {
        _set_selected_widget(_active_widgets.nth_data(index));
    }

    protected uint _get_selection_widget_count() {
        if (_selectable)
            return _active_widgets.length();
        else
            return 0;
    }

    protected int _get_selected_widget_index() {
        if (_currently_selected_widget != null)
            return _active_widgets.index(_currently_selected_widget);
        return - 1;
    }

}

//namespace IndicatorWeather
}

