/* menuitem_table.vala
 *
 * Copyright(C) 2011  Panagiotis Skintzos
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *     Panagiotis Skintzos <panajotis@gmail.com>
 */

using Gtk;

namespace IndicatorWeather {

/**
 * Condition menuitem
 *
 * Subclass of BaseMenuItem
 *
 * It displays a table with arbitrary fields.
 *
 * The table element contents are controlled by the MENUITEM_PROP_FIELDS property.
 * It should be a string formatted as:
 * label1 FIELD_VALUE_SEPARATOR property1 FIELD_SEPARATOR
 * label2 FIELD_VALUE_SEPARATOR property2 FIELD_SEPARATOR
 *
 * This should be set only when it is changed.
 * Then you can adjust the value of a property by setting the propertyN value
 *
 **/
class TableMenuItem : BaseMenuItem {

    /**
     * A map containing the condition field title widgets
     */
    private Gee.HashMap<string, Label> _map_field_titles;
    /**
     * A map containing the condition field value widgets
     */
    private Gee.HashMap<string, Label> _map_field_values;

    /**
     * The internal child
     */
    private Box _box_container;

    /**
     * The table with optional condition fields
     *
     * The plan was to resize the table on the fly and reuse the widgets
     * but somehow dbusmenu causes troubles and label widgets turn to ghosts
     * so we recreate the table and its labels,
     * each time the fields change
     */
    private Table _tbl_fields;

    /**
     * The table parent
     */
    private Alignment _ali_table;

    /**
     * Space between rows
     */
    private int _row_margin = 6;
    /**
     * Space between columns
     */
    private int _col_margin = 5;

    /**
     * Alignment for the value labels
     *  0 : left
     *  1 : right
     */
    private int _alignment = 1;

    /**
     * Attach option constants for expandable table elements
     */
    private static const AttachOptions EXPANDABLE_MASK =
        AttachOptions.FILL | AttachOptions.EXPAND;
    /**
     * Attach option constants for non-expandable table elements
     */
    private static const AttachOptions NON_EXPANDABLE_MASK =
        AttachOptions.FILL;

    /**
     * Hidden constructor
     */
    private TableMenuItem(Dbusmenu.Menuitem item) {
        base(item);
        _selectable = false;
        _dismiss_menu = false;

        _dir = get_default_direction();
        _box_container = new HBox(false, 0);

        _map_field_titles = new Gee.HashMap<string, Label>();
        _map_field_values = new Gee.HashMap<string, Label>();

        // the table parent
        _ali_table = new Alignment(0.5f, 0.5f, 1, 1);

        // we want a padding on the opposite side of the item
        Label margin = new Label(null);
        margin.set_padding(MENU_PADDING, 0);
        if (_dir == TextDirection.RTL) {
            _box_container.pack_start(margin, false);
            _box_container.pack_start(_ali_table);
        } else {
            _box_container.pack_start(_ali_table);
            _box_container.pack_start(margin, false);
        }

        _box_container.show_all();
        this.add(_box_container);
        _update();
    }

    /**
     * Standard construction method called from the dbusmenu client.
     *
     * Creates an instance and registers it.
     */
    public static bool new_menuitem (Dbusmenu.Menuitem newitem,
        Dbusmenu.Menuitem parent, Dbusmenu.Client client) {

        ((DbusmenuGtk.Client)client).newitem_base(newitem,
            new TableMenuItem(newitem), parent);
        return true;
    }

    private void _update() {
        // update the widgets
        if (_dbusitem.property_exist(MENUITEM_PROP_FIELDS))
            _set_field_table(_dbusitem.property_get(MENUITEM_PROP_FIELDS));
        if (_dbusitem.property_exist(MENUITEM_PROP_COL_MARGIN))
            _set_col_margin(_dbusitem.property_get_int(
                MENUITEM_PROP_COL_MARGIN));
        if (_dbusitem.property_exist(MENUITEM_PROP_ROW_MARGIN))
            _set_row_margin(_dbusitem.property_get_int(
                MENUITEM_PROP_ROW_MARGIN));
        if (_dbusitem.property_exist(MENUITEM_PROP_ALIGNMENT))
            _set_alignment(_dbusitem.property_get_int(
                MENUITEM_PROP_ALIGNMENT));

        connect_events();
    }

    public override void on_property_changed (string property, GLib.Variant value) {
        if (_map_field_values.has_key(property)) {
            string markup = value.get_string();
            debug("%s changed %s: %s", name, property, markup);
            Label lbl_value = _map_field_values.get(property);
            lbl_value.set_markup(markup);

        } else if (property == MENUITEM_PROP_FIELDS) {
            _set_field_table(value.get_string());

        } else if (property == MENUITEM_PROP_COL_MARGIN) {
            _set_col_margin(value.get_int32());

        } else if (property == MENUITEM_PROP_ROW_MARGIN) {
            _set_row_margin(value.get_int32());

        } else if (property == MENUITEM_PROP_ALIGNMENT) {
            _set_alignment(value.get_int32());

        } else {
            base.on_property_changed(property, value);
        }
    }

    private void _set_field_table(string? fieldstr) {
        string[] fields;
        if (fieldstr != null) {
            debug("%s: fields: %s", name, fieldstr);
            fields = fieldstr.split(FIELD_SEPARATOR);
        } else {
            fields = new string[]{};
        }

        if (_tbl_fields != null)
            _tbl_fields.destroy();

        // resize to current fields
        debug("%s: resizing table to %dx%d", name, fields.length, 2);
        if (fields.length > 0) {
            _tbl_fields = new Table(fields.length, 2, false);
            _tbl_fields.set_col_spacings(_col_margin);
            _tbl_fields.set_row_spacings(_row_margin);
            _ali_table.add(_tbl_fields);

            // create & attach widgets
            for (int i = 0; i < fields.length; i ++ ) {
                string[] texts = fields[i].split(FIELD_VALUE_SEPARATOR);
                if (texts.length == 2) {
                    debug("%s: adding field %d: %s %s",
                        name, i, texts[0], texts[1]);
                    _create_table_field(texts[0], texts[1], i);
                }
            }
            _box_container.show_all();
        } else {
            _box_container.hide();
        }
    }
    private void _create_table_field(string title, string property, uint row) {

        Label lbl_title = null;
        Label lbl_value = null;
        lbl_title = new Label(title);
        _map_field_titles.set(property, lbl_title);
        lbl_title.set_name("Title %s".printf(property));

        lbl_value = new Label(null);
        _map_field_values.set(property, lbl_value);
        lbl_value.set_name("Value %s".printf(property));
        if (_dbusitem.property_exist(property))
            lbl_value.set_markup(_dbusitem.property_get(property));

        float effective_alignment = (float)_alignment;
        if (_dir == TextDirection.RTL) {
            if (_alignment == 0)
                effective_alignment = 1;
            else if (_alignment == 1)
                effective_alignment = 0;
            _tbl_fields.attach(lbl_value, 0, 1, row, row + 1,
                            NON_EXPANDABLE_MASK, EXPANDABLE_MASK, 0, 0);
            _tbl_fields.attach(lbl_title, 1, 2, row, row + 1,
                            EXPANDABLE_MASK, EXPANDABLE_MASK, 0, 0);
            lbl_title.set_alignment(1, 0.5f);
            lbl_value.set_alignment(effective_alignment, 0.5f);
        } else {
            _tbl_fields.attach(lbl_title, 0, 1, row, row + 1,
                            EXPANDABLE_MASK, EXPANDABLE_MASK, 0, 0);
            _tbl_fields.attach(lbl_value, 1, 2, row, row + 1,
                            NON_EXPANDABLE_MASK, EXPANDABLE_MASK, 0, 0);
            lbl_title.set_alignment(0, 0.5f);
            lbl_value.set_alignment(effective_alignment, 0.5f);
        }
    }
    private void _set_col_margin(int col_margin) {
        debug("%s: col_margin: %d", name, col_margin);
        _col_margin = col_margin;
        if (_tbl_fields != null)
            _tbl_fields.set_col_spacings(_col_margin);
    }
    private void _set_row_margin(int row_margin) {
        debug("%s: row_margin: %d", name, row_margin);
        _row_margin = row_margin;
        if (_tbl_fields != null)
            _tbl_fields.set_row_spacings(_row_margin);
    }
    private void _set_alignment(int alignment) {
        debug("%s: alignment: %d", name, alignment);
        _alignment = alignment;
        if (_alignment < 0)
            _alignment = 0;
        else if (_alignment > 1)
            _alignment = 1;

        if (_tbl_fields != null) {
            float effective_alignment = (float)_alignment;
            if (_dir == TextDirection.RTL) {
                if (_alignment == 0)
                    effective_alignment = 1;
                else if (_alignment == 1)
                    effective_alignment = 0;
            }
            Gee.MapIterator<string, Label> iter =
                _map_field_values.map_iterator();
            while(iter.has_next()) {
                if (iter.next()) {
                    Label lbl_value = iter.get_value();
                    lbl_value.set_alignment(effective_alignment, 0.5f);
                }
            }
        }
    }

}

//namespace IndicatorWeather
}

