#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'ExtendedForecast',
    'Forecast',
    ]

import gtk, pygtk
pygtk.require('2.0')
import urllib2
import pywapi

from indicator_weather_service import *
from .weather import Weather, WeatherSource
from .units import MetricSystem, WindUnit

import gettext
from gettext import gettext as _
gettext.textdomain(GETTEXT_DOMAIN)

class ExtendedForecast(gtk.Window):
    """ Class for forecast window """
    __gtype_name__ = "ExtendedForecast"

    # Create forecast
    def __new__(cls, **kwargs):
        kwargs["log"].debug("ExtendedForecast: creating")
        builder = get_builder('ExtendedForecast')
        new_object = builder.get_object("extended_forecast")
        new_object.finish_initializing(builder, **kwargs)
        return new_object

    def __init__(self, type=gtk.WINDOW_TOPLEVEL, **kwargs):
        gtk.Window.__init__(self)

    # Fill in forecast parameters
    def finish_initializing(self, builder, **kwargs):
        self.log = kwargs.pop("log")
        place = kwargs.pop("place")
        lat = kwargs.pop("lat")
        lon = kwargs.pop("lon")
        metric_system = kwargs.pop("metric_system")
        self.log.debug("ExtendedForecast: finishing initialization")
        self.builder = builder
        self.builder.connect_signals(self)

        # Get forecast data using Forecast object
        self.log.debug("ExtendedForecast: chosen place: %s (code %s)" % (place[1], place[0]))
        self.builder.get_object('extended_forecast').set_title("%s %s" % (_('Weather Forecast for '), place[1]))
        self.log.debug("ExtendedForecast: getting forecast data")
        forecast = Forecast(metric_system, lat, lon,
            get_current_locale_name(), self.log)
        forecast.prepare_forecast_data()
        daysofweek = forecast.get_forecast_daysofweek()
        forecast_data = forecast.get_forecast_data()
        if forecast_data == None:
            # Forecast data unavailable - hide elements and show 'connection_error' label
            self.builder.get_object('connection_error').set_visible(True);
            self.builder.get_object('hbox1').set_visible(False);
            self.builder.get_object('hseparator1').set_visible(False);
            return
        (highdata, lowdata) = forecast_data
        icons      = forecast.get_forecast_icons()
        conditions = forecast.get_forecast_conditions()

        self.log.debug("ExtendedForecast: parsing forecast data")
        # Create labels for each weekday
        self.builder.get_object('day1lbl').set_label('<big>%s</big>' % daysofweek[0].capitalize())
        self.builder.get_object('day2lbl').set_label('<big>%s</big>' % daysofweek[1].capitalize())
        self.builder.get_object('day3lbl').set_label('<big>%s</big>' % daysofweek[2].capitalize())
        self.builder.get_object('day4lbl').set_label('<big>%s</big>' % daysofweek[3].capitalize())

        googlecls = WeatherSource.load_class(WeatherSource.GOOGLE)

        # Fill in icons
        for i in xrange(1,5):
            # Get icon name from dictionary in Weather object for Google icons
            google_icon = googlecls.CONDITIONS[icons[i-1]][0]
            self.builder.get_object('day%simage' % str(i)).set_from_icon_name(google_icon,gtk.ICON_SIZE_BUTTON)

        # Fill in condition labels
        for i in xrange(1,5):
            self.builder.get_object('day%scond' % str(i)).set_label(conditions[i-1])

        # Fill in High and Low temperatures
        if metric_system == MetricSystem.SI:
            tempunit = '°C'
        else:
            tempunit = '°F'
        for i in xrange(1,5):
            label = "%s: %s%s" % (_('High'), highdata[i-1],tempunit)
            self.builder.get_object('day%stemphigh' % str(i)).set_label(label)
            label = "%s: %s%s" % (_('Low'), lowdata[i-1],tempunit)
            self.builder.get_object('day%stemplow' % str(i)).set_label(label)

    def close(self, widget, data=None):
        self.log.debug("ExtendedForecast: closing window")
        self.destroy()

class Forecast:
    """ Class to get forecast information """

    # Initialize a class with metric system, wind units, location and current user locale
    def __init__ (self, units, lat, lon, locale, log):
        self.metric_system = units
        self.lat = lat
        self.lon = lon
        self.locale = locale
        self.log = log

    # Get and store forecast data. For now using Google only
    def prepare_forecast_data(self):
        self.daysofweek = []
        self.icons = []
        self.conditions = []
        try:
            # Generate a fake location by current coordinates
            location_name = ",,,%s,%s" % (self.lat, self.lon)
            self.forecast = pywapi.get_weather_from_google (location_name, hl = self.locale)
            self.log.debug("Forecast: prepare_forecast_data: %s" % self.forecast)
            self.unitsystem = self.forecast['forecast_information']['unit_system']

            for forecast in self.forecast['forecasts']:
                self.daysofweek.append(forecast["day_of_week"])
                self.icons.append(forecast["icon"].split("/ig/images/weather/")[-1].split(".gif")[0])
                self.conditions.append(forecast["condition"])
        #except urllib2.URLError:
        except Exception, e:
            self.log.error("Forecast: error reading forecast for %s" % location_name)

    # Parse high values for forecast data
    def get_forecast_data(self):
        self.highdata = []
        self.lowdata = []

        if not hasattr(self, 'unitsystem'):
            return None

        if ((self.unitsystem == 'SI') and (self.metric_system == MetricSystem.SI)) or ((self.unitsystem == 'US') and (self.metric_system == MetricSystem.IMPERIAL)):
            #correct scale selected
            for forecast in self.forecast['forecasts']:
                self.highdata.append(forecast["high"])
                self.lowdata.append(forecast["low"])

        elif ((self.unitsystem == 'SI') and (self.metric_system == MetricSystem.IMPERIAL)):
            #convert from SI to imperial
            for forecast in self.forecast['forecasts']:
                self.highdata.append(int(((int(forecast["high"])*9)/5)+32))
                self.lowdata.append(int(((int(forecast["low"])*9)/5)+32))

        elif ((self.unitsystem == 'US') and (self.metric_system == MetricSystem.SI)):
            #convert from imperial to SI
            for forecast in self.forecast['forecasts']:
                self.highdata.append(int((((int(forecast["high"]))-32)*5)/9))
                self.lowdata.append(int((((int(forecast["low"]))-32)*5)/9))

        return (self.highdata, self.lowdata)

    # Parse a list of days of week with forecast data
    def get_forecast_daysofweek(self):
        return self.daysofweek

    # Parse icons for forecast data
    def get_forecast_icons(self):
        return self.icons

    # Parse conditions for forecast data
    def get_forecast_conditions(self):
        return self.conditions
