#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'WeatherIndicator',
    ]

__NAME__ = "Weather Indicator"
__VERSION__ = "VERSION HERE"

from indicator_weather_service import *

from gi.repository import Dbusmenu

import glib
import gtk
import traceback
import re
import types
import threading
import pynotify

import gettext
from gettext import gettext as _
gettext.textdomain(GETTEXT_DOMAIN)

from .service import ServiceInterface
from .menu import Menu, MenuItem
from .weather import Weather, WeatherField, WeatherSection
from .settings import Settings, Notification, IndicatorDisplay
from .prefs import PreferencesDialog
from .forecast import ExtendedForecast

class WeatherIndicator(threading.Thread):

    def __init__(self, log):
        """
        Initializes the object
        """
        self.log=log

        # indicator icon & label
        self.indicator_icon = ICON_DEFAULT
        self.indicator_label = "--"
        # refresh label
        self.refreshed_minutes_ago = 0
        # current location's code
        self.current_location_code = None
        # dict with the weather objects (key is location code)
        self.weather = {}
        # severe weather alert info
        self.weather_alert = None
        # dialog dict
        self.dialogs = {}
        # init settings
        self.settings = Settings(self.log)
        # init menu
        self.menu = Menu(self)

    def destroy(self):
        """
        Destroys resources
        """
        self.current_location_code = None
        self.remove_scheduled_jobs()

    def start(self, bus):
        """
        Creates the service interface and the menu,
        and starts the weather updates
        """
        self.service = ServiceInterface(bus, self)
        self.menu.create_menu()
        self.load_settings(True)
        monitor_upower(self.on_system_sleep, self.on_system_resume, self.log)

    def load_settings(self, init = False):
        """
        Reads the current settings and acts if they're changed
        """
        self.log.debug("Indicator: parsing settings")

        changed = self.settings.changed
        need_to_reschedule_update = False
        need_to_update_weather = init
        need_to_update_display = init

        # handle weather objects
        places = self.settings.places
        if places is not None:
            new_codes = [x[0] for x in places]
            # remove old objects
            for key in self.weather.keys():
                if key not in new_codes:
                    self.weather.pop(key)

            for location_code in new_codes:
                ld = self.settings.get_location_details(location_code)
                if location_code not in self.weather:
                    # load new weather object
                    self.weather[location_code] = Weather(location_code,
                        ld, self.settings, self.log)
                    need_to_update_weather = True

                # details might have changed, set them again
                if self.weather[location_code].set_details(ld,
                        self.settings.metric_system,
                        self.settings.wind_unit,
                        self.settings.weather_source):
                    need_to_update_weather = True

            # check placechosen
            if self.settings.placechosen > len(new_codes):
                self.settings.placechosen = 0

        # check changed settings and act accordingly
        if Settings.INDICATOR_DISPLAY in changed:
            self.service.DisplayModeChanged(self.settings.indicator_display)

        if init or Settings.SHOW_FORECAST in changed:
            self.menu.set_forecast_items()
            need_to_update_display = True

        if init or Settings.CONDITION_FIELDS in changed:
            self.menu.set_condition_fields()
            need_to_update_display = True

        if Settings.SHOW_LOCATIONS in changed:
            self.menu.set_location_items()
            need_to_update_display = True

        if Settings.WEATHER_SOURCE in changed:
            need_to_update_weather = True

        if Settings.METRIC_SYSTEM in changed or Settings.WIND_UNIT in changed:
            need_to_update_display = True

        if Settings.REFRESH_RATE in changed:
            need_to_reschedule_update = True

        if init or Settings.PLACES in changed:
            need_to_update_weather = True
            if places is None or len(places) == 0:
                self.menu.set_location_items([])
                self.set_location(None)
            else:
                self.menu.set_location_items(new_codes)
                self.set_location(places[self.settings.placechosen][0], False)

        # clear changes
        self.settings.changed = list()

        # run required updates
        if need_to_update_display or need_to_update_weather:
            self.update_display()
        if self.current_location_code is not None:
            if need_to_update_weather:
                self.update_weather()
            elif need_to_reschedule_update:
                self.schedule_weather_update()

    def set_location(self, code, update_display = True):
        """
        Sets the active location code
        """
        places = self.settings.places
        if self.current_location_code != code:
            old_code = self.current_location_code
            self.log.debug("Indicator: location changed " +
                "from %s to %s" % (old_code, code))
            self.menu.on_location_changed(old_code, code)
            self.current_location_code = code
            if self.current_location_code in [x[0] for x in places]:
                # save placechosen
                for place in places:
                    if place[0] == self.current_location_code:
                        placechosen = places.index(place)
                        self.log.debug("Indicator: new place chosen:"+
                            " %d : %s" % (placechosen, place))
                        self.settings.placechosen = placechosen
                        break
                # update indicator & menu info
                if update_display:
                    self.update_display()

            if self.current_location_code is None:
                self.remove_scheduled_jobs()
                self.update_indicator_icon(ICON_DEFAULT)
                self.menu.update_refresh_label(clear = True)

    def on_system_sleep(self):
        """
        Callback from UPower that system suspends/hibernates
        """
        # store time
        self.sleep_time = datetime.datetime.now()
        self.log.debug("Indicator: system goes to sleep at %s" % self.sleep_time)
        # remove glib timeouts
        self.remove_scheduled_jobs()

    def on_system_resume(self):
        """
        Callback from UPower that system resumes
        """
        now = datetime.datetime.now()
        self.log.debug("Indicator: system resumes at %s" % now)
        # update refresh label
        td = now - self.sleep_time
        mins_elapsed = td.days/24*60 + td.seconds/60 + self.refreshed_minutes_ago
        self.update_refresh_label(mins_elapsed)
        # check if we need to update the weather now or to reschedule the update
        mindiff = self.settings.refresh_rate - mins_elapsed
        if mindiff > 0:
            self.schedule_weather_update(mindiff)
        else:
            self.update_weather()

    def schedule_weather_update(self, rate_override = None):
        """
        Schedules weather update on the preset period or on rate_override
        """
        if hasattr(self, 'weather_update_timeout'):
            glib.source_remove(self.weather_update_timeout)
        if rate_override:
            self.log.debug("Indicator: scheduling update in %s mins" % \
                rate_override)
            self.weather_update_timeout = glib.timeout_add(
                rate_override * 60000, self.update_weather)
        else:
            self.log.debug("Indicator: scheduling update in %s mins" % \
                self.settings.refresh_rate)
            self.weather_update_timeout = glib.timeout_add(
                self.settings.refresh_rate * 60000, self.update_weather)

    def schedule_refresh_label_update(self):
        """
        Schedules refresh label update on the preset period (1min)
        """
        if hasattr(self, 'refresh_update_timeout'):
            glib.source_remove(self.refresh_update_timeout)
        self.log.debug("Indicator: scheduling refresh label update in 1 min")
        self.refresh_update_timeout = glib.timeout_add(60000,
            self.update_refresh_label)

    def remove_scheduled_jobs(self):
        """
        Removes existing scheduled jobs
        """
        if hasattr(self, 'weather_update_timeout'):
            glib.source_remove(self.weather_update_timeout)
        if hasattr(self, 'refresh_update_timeout'):
            glib.source_remove(self.refresh_update_timeout)

    def update_refresh_label(self, reset_minutes = None):
        """
        Updates 'Refresh' menu item display and schedules its update
        """
        if reset_minutes is not None:
            self.refreshed_minutes_ago = reset_minutes
        else:
            self.refreshed_minutes_ago += 1
        self.menu.update_refresh_label()
        self.schedule_refresh_label_update()
        return False

    def update_weather(self):
        """
        Updates the weather for all active locations
        """
        places = self.settings.places
        if self.current_location_code is None:
            return False

        self.menu.update_refresh_label(refreshing = True)

        self.log.debug("Indicator: updating weather")
        # Start a new thread to get new data
        threading.Thread(target = self.do_update_weather,
                         name = 'Fetcher').start()
        return False

    # Get fresh weather data
    def do_update_weather(self):
        failed = False
        try:
            places = self.settings.places
            if self.current_location_code is None:
                return

            succeeded = []
            # Get weather for all locations
            for location_code, location_name in places:
                self.log.debug("Indicator: updating weather report for " + \
                    "%s:'%s'" % (location_code, location_name))
                if self.weather[location_code].update():
                    succeeded.append((location_code, location_name))
                else:
                    failed = True
                    self.log.error("Indicator: could not get weather for " + \
                        "%s:'%s', leaving cached data" % \
                        (location_code, location_name))

            # If we have at least one new weather report, update refresh menuitem
            if len(succeeded):
                self.update_refresh_label(0)

            # Update display elements
            self.update_display()

            # Notifications
            notifications = self.settings.notifications
            global_notifications = self.settings.global_notifications
            for location_code, location_name in places:
                weather = self.weather[location_code]
                condition = weather.get_field_value(WeatherField.CONDITION)
                if condition != MISSING_VALUE:
                    # Check if we send notification
                    if global_notifications or \
                            location_code == self.current_location_code:
                        icon = weather.get_field_value(WeatherField.ICON)
                        severe = weather.get_field_value(WeatherField.SEVERE)

                        if severe and notifications == Notification.ONLY_SEVERE:
                            self.do_notify(location_name, condition, icon, True)

                        elif notifications == Notification.ALL and \
                                condition != \
                                self.settings.get_location_condition(
                                    location_code):
                            self.do_notify(location_name, condition, icon)

                    # Save current condition
                    self.settings.save_location_condition(
                        location_code, condition)

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))

        # If we have at least one failure, repeat an attempt in one minute
        if failed:
            self.schedule_weather_update(1)
        else:
            self.schedule_weather_update()

    def update_display(self):
        try:
            if self.current_location_code is not None:
                for location_code, location_name in self.settings.places:
                    weather = self.weather[location_code]

                    # Update location menuitem
                    self.menu.update_location_item(location_code, weather)

                    # if it's the active location
                    if location_code == self.current_location_code:
                        if weather.section_exists(WeatherSection.CURRENT):
                            icon = weather.get_field_value(WeatherField.ICON)
                            label = weather.get_field_value(
                                WeatherField.TEMPERATURE_SHORT)
                        else:
                            icon = MISSING_VALUE
                            label = MISSING_VALUE
                        self.log.debug("Indicator: update " +
                            "icon '%s' and label '%s'" % (icon, label))
                        self.update_indicator_icon(icon)
                        self.update_indicator_label(label)

                        # Update menuitems
                        self.menu.update_menuitem_data(weather)

            # Update menuitem visibility
            self.menu.update_menu()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))

    def update_indicator_icon(self, icon):
        if icon == MISSING_VALUE:
            icon = "weather-indicator-unknown"
        if self.indicator_icon != icon:
            self.indicator_icon = icon
            if self.service is not None:
                self.service.IconNameChanged(icon)

    def update_indicator_label(self, markup):
        if self.indicator_label != markup:
            self.indicator_label = markup
            if self.service is not None:
                self.service.LabelMarkupChanged(markup)

    def do_notify(self, location_name, condition, icon, severe=False):
        """ Shows a notification to user """
        self.log.debug("Indicator: Notify on weather condition, " +
            "location_name=%s, condition=%s, icon=%s, severe=%s" % \
                (location_name, condition, icon, severe))
        if severe:
            pynotify.Notification ("%s: %s" % (_("Severe weather alert"),
                condition), location_name, icon).show()
        else:
            pynotify.Notification (condition, location_name, icon).show()

    def on_weather_alert(self):
        self.log.debug("Indicator: on_weather_alert")

    def on_refresh(self):
        self.log.debug("Indicator: refresh")
        self.update_weather()

    def on_dialog_destroy(self, dialog):
        key=dialog.get_data('key')
        self.log.debug("Indicator: dialog destroyed: %s" % key)
        if key in self.dialogs:
            self.dialogs.pop(key)

    def on_preferences_response(self, dialog, response):
        self.log.debug("Indicator: Preferences response: %s" % response)
        try:
            dialog.hide()
            if response == gtk.RESPONSE_CLOSE:
                dialog.save_settings()
                dialog.destroy()
                if len(self.settings.changed)>0:
                    self.load_settings()
            else:
                dialog.destroy()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))

    # Open Preferences dialog
    def on_preferences(self):
        self.log.debug("Indicator: open Preferences")
        dlg = None
        dlg_key = MenuItem.PREFERENCES
        try:
            if dlg_key in self.dialogs:
                self.dialogs[dlg_key].present()
            else:
                self.settings.changed = list()
                dlg = PreferencesDialog(**{
                    'log' : self.log,
                    'settings' : self.settings,
                })
                self.dialogs[dlg_key] = dlg
                dlg.set_data('key', dlg_key)
                dlg.connect('response', self.on_preferences_response)
                dlg.connect('destroy', self.on_dialog_destroy)
                dlg.show()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            if dlg is not None:
                dlg.destroy()
            if dlg_key in self.dialogs:
                self.dialogs.pop(dlg_key)

    # Open forecast window
    def on_forecast(self):
        self.log.debug("Indicator: open forecast window")
        dlg = None
        dlg_key = MenuItem.FORECAST_WINDOW
        try:
            if dlg_key in self.dialogs:
                self.dialogs[dlg_key].present()
                return

            location_details = self.settings.get_location_details(
                self.current_location_code)
            dlg = ExtendedForecast(**{
                'log' : self.log,
                'place' : self.settings.places[self.settings.placechosen],
                'metric_system' : self.settings.metric_system,
                'lat' : location_details['latitude'],
                'lon' : location_details['longitude'],
            })
            self.dialogs[dlg_key] = dlg
            dlg.set_data('key', dlg_key)
            dlg.connect('destroy', self.on_dialog_destroy)
            dlg.show()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            if dlg is not None:
                dlg.destroy()
            if dlg_key in self.dialogs:
                self.dialogs.pop(dlg_key)

    # Open About dialog
    def on_about(self):
        self.log.debug("Indicator: open About dialog")
        dlg = None
        dlg_key = MenuItem.ABOUT
        try:
            if dlg_key in self.dialogs:
                self.dialogs[dlg_key].present()
                return

            dlg = gtk.AboutDialog()
            dlg.set_name(_(__NAME__))
            dlg.set_version(__VERSION__)
            dlg.set_website("https://launchpad.net/weather-indicator")
            dlg.set_translator_credits(_("translator-credits"))

            try:
                ifile = open(get_doc_file("AUTHORS"), "r")
                contents=ifile.read().replace('\x0c', '')
                ifile.close()
                dlg.set_copyright(contents)
                dlg.set_authors(re.sub("Copyright\s*\(C\)\s*\d+\s*", "",
                    contents).splitlines())

            except Exception, e:
                self.log.error(e)
                self.log.debug(traceback.format_exc(e))

            try:
                ifile = open(get_doc_file("COPYING"), "r")
                contents=ifile.read().replace('\x0c', '')
                ifile.close()
                dlg.set_license(contents)
                dlg.set_wrap_license(True)

            except Exception, e:
                self.log.error(e)
                self.log.debug(traceback.format_exc(e))

            try:
                dlg.set_logo(gtk.gdk.pixbuf_new_from_file(
                    get_data_file("media", ICON_DEFAULT+".png")))

            except Exception, e:
                self.log.error(e)
                self.log.debug(traceback.format_exc(e))

            self.dialogs[dlg_key]=dlg
            dlg.set_data('key', dlg_key)
            dlg.connect('response', lambda d,r: d.destroy())
            dlg.connect('destroy', self.on_dialog_destroy)
            dlg.show()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            if dlg is not None:
                dlg.destroy()
            if dlg_key in self.dialogs:
                self.dialogs.pop(dlg_key)
