#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'MenuItem',
    'Menu',
    ]

from indicator_weather_service import *

from gi.repository import GLib, Dbusmenu

import traceback
import re
import types
import random
import string

import gettext
from gettext import gettext as _
gettext.textdomain(GETTEXT_DOMAIN)

from .weather import Weather, WeatherField, WeatherSection

# used Dbusmenu constants
MENUITEM_TYPE_DEFAULT           = Dbusmenu.CLIENT_TYPES_DEFAULT
MENUITEM_PROP_TYPE              = Dbusmenu.MENUITEM_PROP_TYPE
MENUITEM_PROP_ENABLED           = Dbusmenu.MENUITEM_PROP_ENABLED
MENUITEM_PROP_VISIBLE           = Dbusmenu.MENUITEM_PROP_VISIBLE
MENUITEM_PROP_TOGGLE_TYPE       = Dbusmenu.MENUITEM_PROP_TOGGLE_TYPE
MENUITEM_PROP_TOGGLE_STATE      = Dbusmenu.MENUITEM_PROP_TOGGLE_STATE
MENUITEM_TOGGLE_RADIO           = Dbusmenu.MENUITEM_TOGGLE_RADIO
MENUITEM_TOGGLE_STATE_UNCHECKED = Dbusmenu.MENUITEM_TOGGLE_STATE_UNCHECKED
MENUITEM_TOGGLE_STATE_CHECKED   = Dbusmenu.MENUITEM_TOGGLE_STATE_CHECKED
MENUITEM_SIGNAL_ITEM_ACTIVATED  = Dbusmenu.MENUITEM_SIGNAL_ITEM_ACTIVATED
MENUITEM_SIGNAL_PROPERTY_CHANGED= Dbusmenu.MENUITEM_SIGNAL_PROPERTY_CHANGED

MENUITEM_BOOL_PROPERTIES = (MENUITEM_PROP_ENABLED,
                            MENUITEM_PROP_VISIBLE,
                            MENUITEM_PROP_SELECTABLE,
                            MENUITEM_PROP_DISMISS_MENU)

MENUITEM_INT_PROPERTIES = ( MENUITEM_PROP_ROW_MARGIN,
                            MENUITEM_PROP_COL_MARGIN,
                            MENUITEM_PROP_ICON_SIZE,
                            MENUITEM_PROP_TOGGLE_STATE,
                            MENUITEM_PROP_INDEX,
                            MENUITEM_PROP_ALIGNMENT)

# Tweak these!
DEFAULT_COL_MARGIN = 3
DEFAULT_ROW_MARGIN = 3
DEFAULT_ICON_SIZE = 44

# Menu items - name constants
class MenuItem:
    """
    Class for available menu items
    """

    # name constants
    SETUP               = "setup"
    ABOUT               = "about"
    PREFERENCES         = "preferences"
    REFRESH             = "refresh"
    FORECAST_WINDOW     = "forecast-window"
    WEATHER_ALERT       = "weather-alert"
    WEATHER_ALERT_SEPARATOR = "weather-alert-separator"
    COMMANDS_SEPARATOR  = "commands-separator"
    LOCATION_SEPARATOR  = "location-separator"
    LOCATION_ITEM       = "location-item"
    LOCATION_SELECTOR   = "location-selector"
    CONDITION_SEPARATOR = "condition-separator"
    CONDITION_EMPTY     = "condition-empty"
    CONDITION_CURRENT   = "condition-current"
    CONDITION_TABLE     = "condition-table"
    CONDITION_FORECAST  = "condition-forecast"
    FORECAST_SEPARATOR  = "forecast-separator"
    FORECAST_DAY1       = "forecast1"
    FORECAST_DAY2       = "forecast2"
    FORECAST_DAY3       = "forecast3"
    SUNDATA             = "sundata"

    # visibility filters
    (
        FILTER_NEVER,           # Always invisible
        FILTER_ALWAYS,          # Always visible
        FILTER_NO_LOCATION,     # Visible when no location is set
        FILTER_ONE_LOCATION,    # Visible when only one location is set
        FILTER_HAVE_LOCATION,   # Visible when at least one location is set
        FILTER_MANY_LOCATIONS,  # Visible when at least two locations are set
        FILTER_WEATHER_ALERT,   # Visible when we have weather alerts
        FILTER_HAVE_NOT_CONDITION_CURRENT,   # Visible when we don't have current condition
        FILTER_HAVE_CONDITION_CURRENT,   # Visible when we have current condition
        FILTER_HAVE_CONDITION_FORECAST,  # Visible when we have forecast condition
        FILTER_HAVE_FORECAST1,  # Visible when we have forecast for day1
        FILTER_HAVE_FORECAST2,  # Visible when we have forecast for day2
        FILTER_HAVE_FORECAST3,  # Visible when we have forecast for day3
    ) = range(0, 13)

    # info dict - menuitem definitions
    # INFO_PROPERTIES: fixed properties, set in creation
    #                   at least MENUITEM_PROP_TYPE should be defined
    # INFO_HANDLER: method to call on item click
    # INFO_FILTERS: visibility filter
    # INFO_MAPPINGS: automatic mappings of properties to weather fields
    # INFO_SECTION: weather report section to get values from
    #
    INFO = {
        # setup weather item
        SETUP : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_LABEL : _("Set Up Weather..."),
            },
            INFO_HANDLER : "on_preferences",
            INFO_FILTER : FILTER_NO_LOCATION,
        },
        # about item
        ABOUT : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_LABEL : _("About..."),
            },
            INFO_HANDLER : "on_about",
            INFO_FILTER : FILTER_HAVE_LOCATION,
        },
        # preferences item
        PREFERENCES : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_LABEL : _("Preferences..."),
            },
            INFO_HANDLER : "on_preferences",
            INFO_FILTER : FILTER_HAVE_LOCATION,
        },
        # refresh item
        REFRESH : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_LABEL : _("Refresh"),
                MENUITEM_PROP_DISMISS_MENU : False,
            },
            INFO_HANDLER : "on_refresh",
            INFO_FILTER : FILTER_HAVE_LOCATION,
        },
        # weather alert item for severe weather warnings
        #TODO: implement this
        WEATHER_ALERT : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_LABEL : _("Warning..."),
            },
            INFO_HANDLER : "on_weather_alert",
            INFO_FILTER : FILTER_WEATHER_ALERT,
        },
        # separator after the weather alert item
        WEATHER_ALERT_SEPARATOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_SEPARATOR,
            },
            INFO_FILTER : FILTER_WEATHER_ALERT,
        },
        # separator at the bottom above the commands
        COMMANDS_SEPARATOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_SEPARATOR,
            },
            INFO_FILTER : FILTER_HAVE_LOCATION,
        },
        # separator at the top of location items
        # used as an anchor to insert a location item
        LOCATION_SEPARATOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_SEPARATOR,
            },
            INFO_FILTER : FILTER_MANY_LOCATIONS,
        },
        # base definition for location items
        LOCATION_ITEM : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_DISMISS_MENU : False,
            },
            INFO_FILTER : FILTER_MANY_LOCATIONS,
            INFO_SECTION : WeatherSection.CURRENT,
            INFO_MAPPINGS : {
                MENUITEM_PROP_LABEL : WeatherField.LOCATION,
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_DETAIL : WeatherField.TEMPERATURE,
            }
        },
        # carousel-like location selection item
        LOCATION_SELECTOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_LOCATION,
            },
            INFO_FILTER : FILTER_HAVE_LOCATION,
            INFO_SECTION : WeatherSection.INFO,
            INFO_MAPPINGS : {
                MENUITEM_PROP_LABEL : WeatherField.LOCATION,
            }
        },
        # separator at the top of current condition
        CONDITION_SEPARATOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_SEPARATOR,
            },
            INFO_FILTER : FILTER_HAVE_LOCATION,
        },
        # item to display when we don't have current condition
        CONDITION_EMPTY : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_SELECTABLE : False,
                MENUITEM_PROP_LABEL : _("No data available"),
            },
            INFO_FILTER : FILTER_HAVE_NOT_CONDITION_CURRENT,
        },
        # current condition item
        CONDITION_CURRENT : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_CONDITION,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_ROW_MARGIN : DEFAULT_ROW_MARGIN,
                MENUITEM_PROP_ICON_SIZE : DEFAULT_ICON_SIZE,
                MENUITEM_PROP_LAYOUT : CONDITION_ITEM_LAYOUT_COMPACT,
            },
            INFO_FILTER : FILTER_HAVE_CONDITION_CURRENT,
            INFO_SECTION : WeatherSection.CURRENT,
            INFO_MAPPINGS : {
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_LABEL : WeatherField.TIMESTAMP,
                MENUITEM_PROP_LABEL2 : WeatherField.CONDITION,
                MENUITEM_PROP_DETAIL : WeatherField.TEMPERATURE,
            }
        },
        # item to display various condition fields
        CONDITION_TABLE : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_TABLE,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_ROW_MARGIN : DEFAULT_ROW_MARGIN,
                MENUITEM_PROP_ALIGNMENT : 1,
            },
            INFO_FILTER : FILTER_HAVE_CONDITION_CURRENT,
            INFO_SECTION : WeatherSection.CURRENT,
        },
        # separator at the top of forecast items
        # used as an anchor to insert a forecast item
        FORECAST_SEPARATOR : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_SEPARATOR,
            },
            INFO_FILTER : FILTER_HAVE_CONDITION_FORECAST,
        },
        # today's forecast item
        CONDITION_FORECAST : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_CONDITION,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_ROW_MARGIN : DEFAULT_ROW_MARGIN,
                MENUITEM_PROP_ICON_SIZE : DEFAULT_ICON_SIZE,
                MENUITEM_PROP_LAYOUT : CONDITION_ITEM_LAYOUT_COMPACT,
            },
            INFO_FILTER : FILTER_HAVE_CONDITION_FORECAST,
            INFO_SECTION : WeatherSection.FORECAST0,
            INFO_MAPPINGS : {
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_LABEL : WeatherField.FORECAST_DAY,
                MENUITEM_PROP_LABEL2 : WeatherField.CONDITION,
                MENUITEM_PROP_DETAIL : WeatherField.TEMP_LOW_HIGH,
            }
        },
        # item to display forecast info for day 1
        FORECAST_DAY1 : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_SELECTABLE : False,
            },
            INFO_FILTER : FILTER_HAVE_FORECAST1,
            INFO_SECTION : WeatherSection.FORECAST1,
            INFO_MAPPINGS : {
                MENUITEM_PROP_LABEL : WeatherField.FORECAST_DAY,
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_DETAIL : WeatherField.TEMP_LOW_HIGH,
            }
        },
        # item to display forecast info for day 2
        FORECAST_DAY2 : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_SELECTABLE : False,
            },
            INFO_FILTER : FILTER_HAVE_FORECAST2,
            INFO_SECTION : WeatherSection.FORECAST2,
            INFO_MAPPINGS : {
                MENUITEM_PROP_LABEL : WeatherField.FORECAST_DAY,
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_DETAIL : WeatherField.TEMP_LOW_HIGH,
            }
        },
        # item to display forecast info for day 3
        FORECAST_DAY3 : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_SELECTABLE : False,
            },
            INFO_FILTER : FILTER_HAVE_FORECAST3,
            INFO_SECTION : WeatherSection.FORECAST3,
            INFO_MAPPINGS : {
                MENUITEM_PROP_LABEL : WeatherField.FORECAST_DAY,
                MENUITEM_PROP_ICON : WeatherField.ICON,
                MENUITEM_PROP_DETAIL : WeatherField.TEMP_LOW_HIGH,
            }
        },
        # forecast window item
        FORECAST_WINDOW : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_LABEL : _("Forecast..."),
            },
            INFO_HANDLER : "on_forecast",
            INFO_FILTER : FILTER_HAVE_CONDITION_CURRENT,
        },
        # item to display together sunrise and sunset
        SUNDATA : {
            INFO_PROPERTIES : {
                MENUITEM_PROP_TYPE : MENUITEM_TYPE_GENERIC,
                MENUITEM_PROP_COL_MARGIN : DEFAULT_COL_MARGIN,
                MENUITEM_PROP_SELECTABLE : False,
                MENUITEM_PROP_LABEL : _("Sun:"),
            },
            INFO_FILTER : FILTER_HAVE_CONDITION_CURRENT,
            INFO_SECTION : WeatherSection.CURRENT,
            INFO_MAPPINGS : {
                MENUITEM_PROP_DETAIL : WeatherField.SUN,
            }
        },
    }

class Menu(object):

    # Menuitem order, bottom to top
    MENUITEM_ORDER = (
        MenuItem.SETUP,
        #MenuItem.ABOUT,
        MenuItem.PREFERENCES,
        MenuItem.REFRESH,
        MenuItem.COMMANDS_SEPARATOR,
        MenuItem.LOCATION_SEPARATOR,
        MenuItem.FORECAST_SEPARATOR,
        #MenuItem.SUNDATA,
        MenuItem.CONDITION_TABLE,
        MenuItem.CONDITION_CURRENT,
        MenuItem.CONDITION_EMPTY,
        MenuItem.CONDITION_SEPARATOR,
        MenuItem.LOCATION_SELECTOR,
        MenuItem.WEATHER_ALERT_SEPARATOR,
        MenuItem.WEATHER_ALERT
    )

    def __init__(self, indicator):
        self.indicator = indicator
        self.log = indicator.log
        self.settings = indicator.settings

        # dbus menu
        self.dbusmenu = None
        # dbus menu root
        self.dbusmenu_root = None
        # dbus menu items
        self.menuitems = {}
        # list that holds the location menuitem keys
        self.location_menuitems = []

    def create_menu(self):
        # register a dbusmenu server
        self.log.info("Menu: registering Dbusmenu.Server: '%s'" % DBUS_MENU_OBJ)
        self.dbusmenu = Dbusmenu.Server.new(DBUS_MENU_OBJ);
        self.dbusmenu_root = Dbusmenu.Menuitem.new()
        self.dbusmenu.set_root(self.dbusmenu_root)

        # initialize the menu
        for key in self.MENUITEM_ORDER:
            self.create_menuitem(key, MenuItem.INFO[key])

        ## handle special items
        #self.set_condition_fields()
        #self.set_forecast_items()

    def on_property_changed(self, menuitem, propname, value, key):
        if propname in MENUITEM_BOOL_PROPERTIES:
            v=menuitem.property_get_bool(propname)
        elif propname in MENUITEM_INT_PROPERTIES:
            v=menuitem.property_get_int(propname)
        else:
            v=menuitem.property_get(propname)
        self.log.debug("Menuitem[%s]: changed '%s': %s" % (key, propname, v))

    def create_menuitem(self, key, iteminfo, position=0, parent=None):
        self.log.debug("Menu: creating menuitem '%s'" % key)
        item = Dbusmenu.Menuitem.new()
        item.property_set(MENUITEM_PROP_KEY, key)
        self.menuitems[key] = item
        self.update_menuitem(key, **{MENUITEM_PROP_VISIBLE : False})
        if INFO_PROPERTIES in iteminfo:
            self.update_menuitem(key, **iteminfo[INFO_PROPERTIES])
        if parent is None:
            self.dbusmenu_root.child_add_position(item, position)
        else:
            parent.child_add_position(item, position)

        # connect handlers
        funcname = iteminfo.get(INFO_HANDLER)
        if funcname is not None:
            if not hasattr(self.indicator, funcname):
                self.log.warning("Indicator: unimplemented method:" +
                    " %s" % funcname)
            else:
                func = getattr(self.indicator, funcname)
                item.connect(MENUITEM_SIGNAL_ITEM_ACTIVATED,
                    lambda x,y,f: f(), func)
        #item.connect(MENUITEM_SIGNAL_PROPERTY_CHANGED,
            #self.on_property_changed, key)

    def delete_menuitem(self, key):
        if key not in self.menuitems:
            return
        self.log.debug("Menu: removing menuitem '%s'" % key)
        self.dbusmenu_root.child_delete(self.menuitems[key])
        self.menuitems.pop(key)

    def update_menuitem(self, key, **properties):
        if key not in self.menuitems:
            return
        item = self.menuitems[key]
        for prop in properties:
            value = properties[prop]
            prop_exists=item.property_exist(prop)
            # boolean properties
            if prop in MENUITEM_BOOL_PROPERTIES:
                #if not prop_exists or item.property_get_bool(prop)!=value:
                #self.log.debug("Changing %s: %s:%s" % (key, prop, value))
                item.property_set_bool(prop, value)
                self.indicator.service.MenuitemPropertyChanged(key, prop, value)

            # int properties
            elif prop in MENUITEM_INT_PROPERTIES:
                #if not prop_exists or item.property_get_int(prop)!=value:
                #self.log.debug("Changing %s: %s:%s" % (key, prop, value))
                item.property_set_int(prop, value)
                self.indicator.service.MenuitemPropertyChanged(key, prop, value)

            # list properties
            elif isinstance(value, types.ListType):
                item.property_set_variant(prop, GLib.Variant.new_strv(value))
                self.indicator.service.MenuitemPropertyChanged(key, prop, value)

            # string properties
            else:
                #if not prop_exists or item.property_get(prop)!=str(value):
                #self.log.debug("Changing %s: %s:%s" % (key, prop, str(value)))
                item.property_set(prop, str(value))
                self.indicator.service.MenuitemPropertyChanged(key, prop, value)

    def update_menu(self):
        self.log.debug("Menu: updating visibility")

        # calculate visibility filters
        loc_cnt = len(self.location_menuitems)
        weather = self.indicator.weather.get(
            self.indicator.current_location_code)
        sections = weather.get_sections() \
            if weather is not None else []

        filters = {
            MenuItem.FILTER_ALWAYS          : True,
            MenuItem.FILTER_NEVER           : False,
            MenuItem.FILTER_NO_LOCATION     : loc_cnt == 0,
            MenuItem.FILTER_ONE_LOCATION    : loc_cnt == 1,
            MenuItem.FILTER_HAVE_LOCATION   : loc_cnt > 0,
            MenuItem.FILTER_MANY_LOCATIONS  : loc_cnt > 1,
        }
        filters[MenuItem.FILTER_HAVE_CONDITION_CURRENT] = \
            filters[MenuItem.FILTER_HAVE_LOCATION] and \
            WeatherSection.CURRENT in sections
        filters[MenuItem.FILTER_HAVE_NOT_CONDITION_CURRENT] = \
            filters[MenuItem.FILTER_HAVE_LOCATION] and \
            WeatherSection.CURRENT not in sections
        filters[MenuItem.FILTER_HAVE_CONDITION_FORECAST] = \
            filters[MenuItem.FILTER_HAVE_CONDITION_CURRENT] and \
            WeatherSection.FORECAST0 in sections
        filters[MenuItem.FILTER_HAVE_FORECAST1] = \
            filters[MenuItem.FILTER_HAVE_CONDITION_FORECAST] and \
            WeatherSection.FORECAST1 in sections
        filters[MenuItem.FILTER_HAVE_FORECAST2] = \
            filters[MenuItem.FILTER_HAVE_FORECAST1] and \
            WeatherSection.FORECAST2 in sections
        filters[MenuItem.FILTER_HAVE_FORECAST3] = \
            filters[MenuItem.FILTER_HAVE_FORECAST2] and \
            WeatherSection.FORECAST3 in sections
        filters[MenuItem.FILTER_WEATHER_ALERT] = \
            filters[MenuItem.FILTER_HAVE_CONDITION_CURRENT] and \
            self.indicator.weather_alert is not None

        # show/hide menuitems depending on their filter
        for key in self.menuitems:
            if key in MenuItem.INFO:
                f = MenuItem.INFO[key].get(INFO_FILTER, MenuItem.FILTER_NEVER)
                # this needs special handling
                if key == MenuItem.LOCATION_SEPARATOR:
                    self.show_menuitem(key,
                        self.settings.show_locations and filters[f])
                # this needs special handling
                elif key == MenuItem.FORECAST_SEPARATOR:
                    self.show_menuitem(key,
                        self.settings.show_forecast and filters[f])
                else:
                    self.show_menuitem(key, filters[f])

            # location items
            elif key in self.location_menuitems:
                f = MenuItem.INFO[MenuItem.LOCATION_ITEM].get(
                    INFO_FILTER, MenuItem.FILTER_MANY_LOCATIONS)
                self.show_menuitem(key,
                    self.settings.show_locations and \
                    self.indicator.current_location_code != key and \
                    filters[f])

    def show_menuitem(self, key, show):
        #self.log.debug("Menu: item %s visible %s" % (key, show))
        self.update_menuitem(key, **{MENUITEM_PROP_VISIBLE : show})

    def toggle_menuitem(self, key, checked):
        self.update_menuitem(key, **{ MENUITEM_PROP_TOGGLE_STATE :
            MENUITEM_TOGGLE_STATE_CHECKED if checked else
            MENUITEM_TOGGLE_STATE_UNCHECKED})

    def update_menuitem_data(self, weather):
        for key in self.menuitems:
            section = WeatherSection.CURRENT
            props = None
            if key == MenuItem.CONDITION_TABLE:
                if weather.section_exists(section):
                    props = dict(map(lambda propname:
                        (MENUITEM_PROP_PREFIX + propname,
                        weather.get_field_value(propname)),
                        self.settings.condition_fields))
            else:
                mappings = None
                if key in MenuItem.INFO:
                    mappings = MenuItem.INFO[key].get(INFO_MAPPINGS)
                    section = MenuItem.INFO[key].get(INFO_SECTION, section)

                if mappings is not None and (weather.section_exists(section) or \
                        (key == MenuItem.LOCATION_SELECTOR)):
                    props = dict(map(lambda propname:
                        (propname, weather.get_field_value(
                        mappings[propname], section)), mappings.iterkeys()))

            if props is not None:
                self.log.debug("Menu: updating menuitem " + \
                    "'%s' with properties: %s" % (key, props))
                self.update_menuitem(key, **props)

    def update_refresh_label(self, refreshing = False):
        if refreshing:
            props = { MENUITEM_PROP_LABEL : _("Refreshing"),
                      MENUITEM_PROP_DETAIL : _("please wait") }

        elif self.indicator.refreshed_minutes_ago < 0:
            props = { MENUITEM_PROP_LABEL : _("Refresh"),
                      MENUITEM_PROP_DETAIL : "" }

        elif self.indicator.refreshed_minutes_ago == 0:
            props = { MENUITEM_PROP_LABEL : _("Refresh"),
                      MENUITEM_PROP_DETAIL : _("just now") }

        else:
            props = { MENUITEM_PROP_LABEL : _("Refresh"),
                      MENUITEM_PROP_DETAIL : _("%d min. ago") % \
                        self.indicator.refreshed_minutes_ago }
        self.log.debug("Indicator: updating refresh label: %s" % props)
        self.update_menuitem(MenuItem.REFRESH, **props)

    def update_location_item(self, key, weather):
        if not self.settings.show_locations or key not in self.menuitems:
            return
        mappings = MenuItem.INFO[MenuItem.LOCATION_ITEM].get(INFO_MAPPINGS)
        section = WeatherSection.CURRENT
        if mappings is not None: # and weather.section_exists(section):
            props = dict(map(lambda propname:
                (propname, weather.get_field_value(mappings[propname])),
                mappings.iterkeys()))
            self.log.debug("Menu: updating menuitem " + \
                "'%s' with properties: %s" % (key, props))
            self.update_menuitem(key, **props)

    def set_condition_fields(self):
        if MenuItem.CONDITION_TABLE not in self.menuitems:
            return
        field_template = "%s:" + FIELD_VALUE_SEPARATOR + \
                                 MENUITEM_PROP_PREFIX + "%s"
        field_list = [field_template % (Weather.get_field_label(x), x)
                        for x in self.settings.condition_fields]
        table_fields = FIELD_SEPARATOR.join(field_list)
        self.log.debug("Menu: setting condition fields to \n%s" % table_fields)
        self.update_menuitem(MenuItem.CONDITION_TABLE, **{
            MENUITEM_PROP_FIELDS : table_fields })

    def set_forecast_items(self):
        # delete old items
        for key in (MenuItem.CONDITION_FORECAST,
                    MenuItem.FORECAST_DAY1,
                    MenuItem.FORECAST_DAY2,
                    MenuItem.FORECAST_DAY3,
                    MenuItem.FORECAST_WINDOW):
            self.delete_menuitem(key)

        # add new items
        position = self.menuitems[
            MenuItem.FORECAST_SEPARATOR].get_position(
                self.dbusmenu_root)
        if self.settings.show_forecast:
            for key in (MenuItem.CONDITION_FORECAST,
                        MenuItem.FORECAST_DAY1,
                        MenuItem.FORECAST_DAY2,
                        MenuItem.FORECAST_DAY3):
                position += 1
                self.create_menuitem(key, MenuItem.INFO[key], position)
        else:
            key = MenuItem.FORECAST_WINDOW
            position += 1
            self.create_menuitem(key, MenuItem.INFO[key], position)

    def set_location_items(self, new_codes = None):

        # delete old items
        for key in self.location_menuitems:
            self.delete_menuitem(key)

        # if codes changed
        if new_codes is not None:
            self.location_menuitems = new_codes
            if len(self.location_menuitems)>0:
                # update location selector
                props = { MENUITEM_PROP_ITEMS : self.location_menuitems,
                          MENUITEM_PROP_INDEX : self.settings.placechosen }
                self.log.debug("Menu: updating menuitem " + \
                    "'%s' with properties: %s" % (
                    MenuItem.LOCATION_SELECTOR, props))
                self.update_menuitem(MenuItem.LOCATION_SELECTOR, **props)

        # add new items
        if self.settings.show_locations:
            position = self.menuitems[
                MenuItem.LOCATION_SEPARATOR].get_position(self.dbusmenu_root)
            for key in self.location_menuitems:
                position += 1
                self.create_menuitem(key,
                    MenuItem.INFO[MenuItem.LOCATION_ITEM], position)
                self.menuitems[key].connect(MENUITEM_SIGNAL_ITEM_ACTIVATED,
                    lambda m,t,key: self.indicator.set_location(key), key)

    def on_location_changed(self, old_code, new_code):
        if self.settings.show_locations:
            if old_code in self.location_menuitems:
                self.update_menuitem(old_code, **{
                    MENUITEM_PROP_VISIBLE : False,
                    MENUITEM_PROP_SELECTABLE : True })

            if new_code in self.location_menuitems:
                self.update_menuitem(new_code, **{
                    MENUITEM_PROP_VISIBLE : True,
                    MENUITEM_PROP_SELECTABLE : False })

        if new_code in self.location_menuitems:
            self.update_menuitem(MenuItem.LOCATION_SELECTOR, **{
                MENUITEM_PROP_INDEX :
                self.location_menuitems.index(new_code) })
