#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'PreferencesDialog',
    'Assistant',
    ]

import traceback
import gtk, pygtk
pygtk.require('2.0')
import urllib2
import urllib
import types

from indicator_weather_service import *
from .settings import Settings, Notification
from .location import Location
from .weather import WeatherField, WeatherSource
from .units import MetricSystem, WindUnit

import gettext
from gettext import gettext as _
gettext.textdomain(GETTEXT_DOMAIN)

class PreferencesDialog(gtk.Dialog):
    """
    Class for preferences dialog
    """
    __gtype_name__ = "PreferencesDialog"

    def __new__(cls, **kwargs):
        """
        Creates a new preferences dialog
        """
        log = kwargs.get('log')
        log.debug("Preferences: creating")
        builder = get_builder('PreferencesDialog')
        new_object = builder.get_object('preferences_dialog')
        new_object.builder = builder
        new_object.finish_initializing(builder, **kwargs)
        return new_object

    def __init__(self, *args, **kwargs):
        """
        Initializes the dialog
        This is needed for the kwargs to work in __new__
        """
        gtk.Dialog.__init__(self, *args)

    def finish_initializing(self, builder, **kwargs):
        """
        Fills in preferences dialog with currect data
        """
        self.assistant = None
        self.builder = builder
        self.log = kwargs.pop('log')
        self.settings = kwargs.pop('settings')
        self.log.debug("Preferences: finishing initialization")
        #all_condition_fields = WeatherField.get_fields_by_flag(
            #WeatherField.FLAG_CONDITION)

        # init widgets with values
        for key in self.settings:
            info = Settings.INFO[key]
            value = self.settings[key]
            self.log.debug("Preferences: loading setting '%s'='%s'" % \
                (key, value))

            if key == Settings.CONDITION_FIELDS:
                # condition fields
                liststore = self.builder.get_object('conditionfields_liststore')
                for key in WeatherField.ALL_CONDITION_FIELDS:
                    info = WeatherField.INFO.get(key)
                    if info is not None:
                        liststore.append([
                            key,                        # code
                            info.get(INFO_LABEL, key),  # label
                            key in value                # show?
                        ])

            elif key == Settings.PLACES:
                # locations
                liststore = self.builder.get_object('locations_liststore')
                if value != None:
                    for location_key, location_name in value:
                        liststore.append([
                            location_key,
                            location_name,
                            self.settings.get_location_details(location_key)
                        ])

            elif INFO_CLASS in info:
                # radio groups
                cls = Settings.INFO[key][INFO_CLASS]
                if INFO_WIDGET in cls.INFO[value]:
                    objname = cls.INFO[value][INFO_WIDGET]
                    self.builder.get_object(objname).set_active(True)

            elif INFO_WIDGET in info:
                objname = info[INFO_WIDGET]
                # checkboxes
                if info[INFO_TYPE] == types.BooleanType:
                    self.builder.get_object(objname).set_active(value)
                # spinners
                elif info[INFO_TYPE] == types.IntType:
                    self.builder.get_object(objname).set_value(float(value))

        objname = Settings.INFO[Settings.GLOBAL_NOTIFICATIONS][INFO_WIDGET]
        self.builder.get_object(objname).set_sensitive(
            self.settings.notifications != Notification.NONE)

        # signals
        self.builder.connect_signals(self)
        self.connect('destroy', self.on_destroy)

    def save_settings(self):
        """
        Saves settings from widget values
        """
        self.log.debug("Preferences: saving settings")

        # save widget values back to settings
        for key in self.settings:
            info = Settings.INFO[key]
            self.log.debug("Preferences: saving setting '%s'" % (key))

            if key == Settings.CONDITION_FIELDS:
                # condition fields
                liststore = self.builder.get_object('conditionfields_liststore')
                treeiter = liststore.get_iter_first()
                new_list = list()
                while (treeiter != None):
                    # if show is checked, grab the code
                    if liststore.get_value(treeiter, 2):
                        new_list.append(liststore.get_value(treeiter, 0))
                    treeiter = liststore.iter_next(treeiter)
                self.settings.condition_fields = new_list

            elif key == Settings.PLACES:
                # locations
                liststore = self.builder.get_object('locations_liststore')
                treeiter = liststore.get_iter_first()
                new_list = list()
                while (treeiter != None):
                    new_list.append([
                        liststore.get_value(treeiter, 0),
                        liststore.get_value(treeiter, 1) ])
                    treeiter = liststore.iter_next(treeiter)

                # if changed we need to keep the current placechosen
                old_list = self.settings.places
                if new_list != old_list:
                    current_place = None
                    if old_list is not None and len(old_list) > 0:
                        current_place = old_list[self.settings.placechosen]
                    if current_place is not None and current_place in new_list:
                        placechosen = new_list.index(current_place)
                    else:
                        placechosen = 0
                    self.settings.places = new_list
                    self.settings.placechosen = placechosen

            elif INFO_CLASS in info:
                # radio groups
                cls = Settings.INFO[key][INFO_CLASS]
                for value in cls.INFO:
                    objname = cls.INFO[value].get(INFO_WIDGET)
                    if objname is not None and \
                            self.builder.get_object(objname).get_active():
                        self.settings[key] = value
                        break

            elif INFO_WIDGET in info:
                objname = info[INFO_WIDGET]
                # checkboxes
                if info[INFO_TYPE] == types.BooleanType:
                    self.settings[key] = \
                        self.builder.get_object(objname).get_active()
                # spinners
                elif info[INFO_TYPE] == types.IntType:
                    self.settings[key] = \
                        int(self.builder.get_object(objname).get_value())

    def on_notifications_toggled(self, widget):
        """
        Callback for notifications radio group toggle
        Need to enable/disable the global_notifications checkbox
        """
        notifications = self.settings.notifications
        for value in Notification.INFO:
            objname = Notification.INFO[value].get(INFO_WIDGET)
            if objname is not None and \
                    self.builder.get_object(objname).get_active():
                notifications = value
                break

        objname = Settings.INFO[Settings.GLOBAL_NOTIFICATIONS][INFO_WIDGET]
        self.builder.get_object(objname).set_sensitive(
            notifications != Notification.NONE)

    def on_field_show_renderer_toggled(self, renderer, path_string):
        """
        Callback for condition_fields checkbox toggle
        Need to set the value in the model
        """
        liststore = self.builder.get_object('conditionfields_liststore')
        treeiter = liststore.get_iter_from_string(path_string)
        value = liststore.get_value(treeiter, 2)
        liststore.set_value(treeiter, 2, not value)

    def on_remove_location(self, widget):
        """
        Callback from Remove location button
        Removes a location from the list
        """
        model, treeiter = self.builder.get_object(
            'locations_view').get_selection().get_selected()
        if treeiter != None:
            self.log.debug("Preferences: removing location [%s] %s" % \
                (model[treeiter][0], model[treeiter][1]))
            model.remove(treeiter)

    def on_add_location(self, widget):
        """
        Callback from Add location button
        Creates a new Assistant to add a new location
        """
        self.log.debug("Preferences: add location clicked")
        if self.assistant is not None:
            if self.assistant.get_visible():
                self.assistant.present()
            else:
                self.assistant.destroy()
                self.assistant = None

        if self.assistant is None:
            self.assistant = Assistant(**{
                'prefswindow': self, 'log': self.log})
            self.assistant.set_parent_window(self.window)
            self.assistant.set_destroy_with_parent(True)
            self.assistant.connect('destroy', self.on_assistant_destroy)
            self.assistant.show()

    def on_destroy(self, dialog):
        """
        Destroy callback
        Destroys the Assistant
        """
        self.log.debug("Preferences: destroying")
        if self.assistant is not None:
            self.assistant.destroy()

    def on_assistant_destroy(self, dialog):
        """
        Assistant destroy callback
        Clears the Assistant reference
        """
        self.assistant = None

    def append_location(self, location_code, location_label, location_details):
        """
        Called from Assistant to add a new location
        Appends the location to the model
        """
        self.settings.save_location_details(
            str(location_code), eval(str(location_details)))
        self.builder.get_object('locations_liststore').append([
            str(location_code),
            location_label,
            str(location_details)])

class Assistant(gtk.Assistant):
    """ Class for a wizard, which helps to add a new location in location list """
    __gtype_name__ = "Assistant"

    # Create new object
    def __new__(cls, **kwargs):
        kwargs['log'].debug("Assistant: creating new Assistance instance")
        builder = get_builder('Assistant')
        new_object = builder.get_object('assistant')
        new_object.finish_initializing(builder, **kwargs)
        return new_object

    def __init__(self, **kwargs):
        gtk.Assistant.__init__(self)

    # Finish UI initialization - prepare combobox
    def finish_initializing(self, builder, **kwargs):
        self.prefswindow = kwargs.pop('prefswindow')
        self.log = kwargs.pop('log')
        self.log.debug("Assistant: finishing initialization")
        self.builder = builder
        self.builder.connect_signals(self)
        self.assistant = self.builder.get_object('assistant')
        self.assistant.set_page_complete(self.builder.get_object('label'),True)
        self.assistant.set_page_complete(self.builder.get_object('review'),True)

        # Set up combobox
        self.log.debug("Assistant: setting up location combobox")
        self.store = gtk.ListStore(str, str, str, str, str)
        self.location_input_combo = self.builder.get_object('combolocations')
        self.location_input_combo.set_model(self.store)
        self.location_input_combo.set_text_column(0)
        self.location_entry = self.builder.get_object('entrylocation')

        self.assistant.set_forward_page_func(self.next_page)

    # 'Get cities' button clicked - get suggested cities list
    def on_get_city_names(self, widget):
        new_text = self.location_entry.get_text()
        self.log.debug("Assistant: looking for location '%s'" % new_text)
        try:
            # Clear up exising suggestions
            self.store.clear()
            # Get suggested city names from GeoNames DB in native locale
            new_text = urllib.quote(new_text)
            url = 'http://api.geonames.org/searchJSON?q=%s&featureClass=P&maxRows=10&lang=%s&username=indicatorweather' % (new_text, get_current_locale_name())
            cities = eval(urllib2.urlopen(url).read())
            for city in cities['geonames']:
                # Create a full city name, consisting of city name, administrative areas names and country name
                if 'adminName2' in city:
                    displayed_city_name = "%s, %s, %s, %s" % (city['name'], city['adminName1'], city['adminName1'], city['countryName'])
                elif 'adminName1' in city:
                    displayed_city_name = "%s, %s, %s" % (city['name'], city['adminName1'], city['countryName'])
                else:
                    displayed_city_name = "%s, %s" % (city['name'], city['countryName'])
                self.store.append([displayed_city_name, str(city['geonameId']), str(city['lat']), str(city['lng']), str(city['name'])])
        except urllib2.URLError:
            self.log.error("Assistant: error reaching url '%s'" % url)

    # A city is selected from suggested list
    def on_select_city(self, entry):
        if self.location_input_combo.get_active() != -1:
            self.place_selected = self.store[self.location_input_combo.get_active()]
            self.assistant.set_page_complete(self.builder.get_object('placeinput'),True)

    # Create a location object out of a selected location
    def next_page(self, current_page):
        try:
            self.log.debug("Assistant: moved to page %s" % current_page)
            if (self.assistant.get_current_page() == 0) and \
                    (not hasattr(self, 'location')) and \
                    (hasattr(self, 'place_selected')):
                # Label input page
                self.log.debug("Assistant: Page %s: got location with code %s" % (current_page, self.place_selected[1]))
                self.location = Location(None, self.log)
                if self.location.prepare_location(self.place_selected):
                    self.log.debug("Assistant: Page %s: City %s found" % (current_page, self.place_selected[0]))
                    # Set a short city name as default label
                    self.builder.get_object('entrylbl').set_text(self.place_selected[4])
                else:
                    self.log.error("Assistant: Page %s: City with code %s was NOT found" % (current_page, self.place_selected[0]))
                    return 3

            elif self.assistant.get_current_page() == 1:
                # Confirmation page
                lbl = self.builder.get_object('entrylbl').get_text()
                self.log.debug("Assistant: Page %s: City label is %s" % (current_page, lbl))
                # If empty label was input, set label to short city name
                if lbl == '':
                    self.log.debug("Assistant: Page %s: Empty label found, setting lbl to short name - %s" % (current_page, self.place_selected[4]))
                    lbl = self.place_selected[4]
                self.location.location_details['label'] = lbl
                self.builder.get_object('lbl3').set_label(_('Label:'))
                self.builder.get_object('labellbl').set_label('<b>%s</b>' % lbl)
                self.builder.get_object('placelbl').set_label('<b>%s</b>' % self.place_selected[0])

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            return self.assistant.get_current_page()

        return self.assistant.get_current_page() + 1

    # 'Cancel' clicked
    def on_cancel(self,widget):
        self.log.debug("Assistant: Cancelled")
        self.destroy()

    # 'Apply' clicked - save location details, add an entry in a location list
    def on_apply(self,widget):
        try:
            (location_code, location_details) = self.location.export_location_details()
            self.log.debug("Assistant: Apply: adding location ('%s', '%s')" % (location_code, location_details))
            self.prefswindow.append_location(location_code, self.location.location_details['label'], location_details)
            self.log.debug("Assistant: finished")

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))

        finally:
            self.hide()
