#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'GoogleWeather',
    ]

import traceback
import re
import datetime
import locale
import pywapi
import calendar
import urllib2
from xml.dom.minidom import parseString

from indicator_weather_service import *
from indicator_weather_service.providers import WeatherProvider
from indicator_weather_service.weather import WeatherField, WeatherSection
from indicator_weather_service.units import MetricSystem, WindUnit

import gettext
from gettext import gettext as _
from gettext import ngettext as __
gettext.textdomain(GETTEXT_DOMAIN)

class GoogleWeather(WeatherProvider):
    """
    WeatherProvider subclass for Google weather data source
    """
    NAME = "Google"

    #Available conditions by google icon
    #Format: Google icon name: (day icon, night icon, is a severe weather condition)
    CONDITIONS = {
        "sunny"            : ( "weather-clear",      "weather-clear-night",      False),
        "mostly_sunny"     : ( "weather-clear",      "weather-clear-night",      False),
        "partly_cloudy"    : ( "weather-few-clouds", "weather-few-clouds-night", False),
        "mostly_cloudy"    : ( "weather-overcast",   "weather-overcast",         False),
        "cloudy"           : ( "weather-clouds",     "weather-clouds-night",     False),
        "rain"             : ( "weather-showers",    "weather-showers",          False),
        "chance_of_rain"   : ( "weather-showers",    "weather-showers",          False),
        "sleet"            : ( "weather-snow",       "weather-snow",             False),
        "rain_snow"        : ( "weather-snow",       "weather-snow",             False),
        "snow"             : ( "weather-snow",       "weather-snow",             False),
        "chance_of_snow"   : ( "weather-snow",       "weather-snow",             False),
        "icy"              : ( "weather-snow",       "weather-snow",             False),
        "flurries"         : ( "weather-snow",       "weather-snow",             False),
        "dust"             : ( "weather-fog",        "weather-fog",              False),
        "fog"              : ( "weather-fog",        "weather-fog",              False),
        "mist"             : ( "weather-fog",        "weather-fog",              False),
        "smoke"            : ( "weather-fog",        "weather-fog",              False),
        "haze"             : ( "weather-fog",        "weather-fog",              False),
        "chance_of_storm"  : ( "weather-storm",      "weather-storm",            True),
        "storm"            : ( "weather-storm",      "weather-storm",            True),
        "thunderstorm"     : ( "weather-storm",      "weather-storm",            True),
        "chance_of_tstorm" : ( "weather-storm",      "weather-storm",            True),
    }

    @staticmethod
    def convert_coordinate(value):
        """
        Converts a coordinate for Google needs
        """
        return int(round(float(value) * 1e6))

    @staticmethod
    def get_location_id(location_code, location_details, log):
        """
        Returns a location id for a specific location
        """
        try:
            lat = GoogleWeather.convert_coordinate(location_details['latitude'])
            lon = GoogleWeather.convert_coordinate(location_details['longitude'])
            return ",,,%s,%s" % (lat, lon)

        except Exception, e:
            log.error(e)
            log.debug(traceback.format_exc(e))
        return None

    def __init__(self, location_id, lat, lon, log):
        """
        Initialize
        """
        super(GoogleWeather, self).__init__(location_id, lat, lon, log)

    def fetch_data(self):
        """
        Gets fresh weather data
        """
        # get data in english locale, imperial units
        self.__report = pywapi.get_weather_from_google(
            self._location_id, hl = 'en')
        #self.log.debug("GoogleWeather: report for %s:\n%s"  % \
            #(self._location_id, self.__report))
        # get data in current locale
        self.__localized_report = pywapi.get_weather_from_google (
            self._location_id, hl = get_current_locale_name())
        #self.log.debug("GoogleWeather: localized report for %s:\n%s"  % \
            #(self._location_id, self.__localized_report))
        # parse data
        return self.__parse_data()

    def __parse_data(self):
        """
        Parses the received data and fills in a dict
        """
        data = {
            WeatherSection.INFO : {
                WeatherField.SOURCE : self.NAME,
                WeatherField.METRIC_SYSTEM : MetricSystem.IMPERIAL,
                WeatherField.WIND_UNIT : WindUnit.MPH,
            }
        }

        item = {}

        # store timestamp
        item[WeatherField.TIMESTAMP] = self._get_timestamp()

        # get sunrise/sunset times
        self._night = False
        item[WeatherField.SUNRISE], item[WeatherField.SUNSET] = \
            self.__get_sunrise_sunset()
        if item[WeatherField.SUNRISE] is not None:
            # calculate whether it is night or day
            now = datetime.datetime.now().time()
            if now < item[WeatherField.SUNRISE] or \
                    now > item[WeatherField.SUNSET]:
                self._night = True
            else:
                self._night = False

        # get condition text, icon, severe
        if 'current_conditions' not in self.__localized_report:
            self.log.warning("GoogleWeather: Missing section: current_conditions")
        else:
            section = self.__localized_report['current_conditions']
            icon_name = self._extract_icon_name(section)
            if icon_name in self.CONDITIONS:
                cond = self.CONDITIONS[icon_name]
                item[WeatherField.ICON] = cond[1] if self._night else cond[0]
                item[WeatherField.SEVERE] = cond[2]

            item[WeatherField.CONDITION] = self._extract_str(
                section, 'condition')

            item[WeatherField.TEMPERATURE] = self._extract_float(
                section, 'temp_f')

        # get humidity, wind
        if 'current_conditions' not in self.__report:
            self.log.warning("GoogleWeather: Missing section: current_conditions")
        else:
            section = self.__report['current_conditions']
            humidity = self._extract_str(section, 'humidity')
            if humidity is not None:
                m = re.match("Humidity:\s*([\d\.\,]+)\s*%", humidity)
                if m is not None:
                    item[WeatherField.HUMIDITY] = float(m.group(1))

            wind = self._extract_str(section, 'wind_condition')
            if wind is not None:
                m = re.match("Wind:\s*([NSEW]{1,2})\s*at\s*([\d\.\,]+)\s*mph", wind)
                if m is not None:
                    item[WeatherField.WIND_SPEED] = float(m.group(2))
                    item[WeatherField.WIND_DIRECTION] = m.group(1)

        # store current conditions
        data[WeatherSection.CURRENT] = item

        # get forecast
        if 'forecasts' not in self.__report:
            self.log.warning("GoogleWeather: Missing section: forecasts")
        else:
            cal_days = self._get_calendar_days()
            for index, section in enumerate(self.__report['forecasts']):
                section_key = "forecast%d" % index
                item = {}
                d = self._extract_str(section, 'day_of_week')
                if d in cal_days:
                    item[WeatherField.FORECAST_DAY] = cal_days[d]

                icon_name = self._extract_icon_name(section)
                if icon_name in self.CONDITIONS:
                    cond = self.CONDITIONS[icon_name]
                    item[WeatherField.ICON] = cond[0]
                    item[WeatherField.SEVERE] = cond[2]

                item[WeatherField.CONDITION] = self._extract_str(
                    section, 'condition')

                item[WeatherField.TEMP_LOW_HIGH] = (
                    self._extract_float(section, 'low'),
                    self._extract_float(section, 'high'))

                data[section_key] = item

            #get localized condition text
            for index, section in enumerate(self.__localized_report['forecasts']):
                section_key = "forecast%d" % index
                data[section_key][WeatherField.CONDITION] = self._extract_str(
                    section, 'condition')

        return data

    def _get_calendar_days(self):
        days = {}
        locale.setlocale(locale.LC_ALL, 'C')
        # need to reset the locale in case of anything goes wrong
        try:
            today = datetime.datetime.now()
            day_list=list(calendar.day_abbr)
            today_index = day_list.index(today.strftime("%a"))
            for index, day in enumerate(day_list):
                if index >= today_index:
                    diff_days = index - today_index
                else:
                    diff_days = 7 - today_index + index
                days[day] = today + datetime.timedelta(days = diff_days)

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))

        locale.resetlocale()
        return days

    def _extract_icon_name(self, section):
        icon_name = self._extract_str(section, 'icon')
        if icon_name is not None:
            m = re.match("/ig/images/weather/(.+)\.gif", icon_name)
            if m is not None:
                icon_name = m.group(1)
        return icon_name

    def __get_sunrise_sunset(self):
        """
        Returns sunrise and sunset times in a tuple
        """
        # Grab data from earthtools.org
        now = datetime.datetime.now()
        url = 'http://www.earthtools.org/sun/%s/%s/%s/%s/99/1' % \
            (self._lat, self._lon, now.day, now.month)
        try:
            f = urllib2.urlopen(url)
            s=f.read()
            parsed = parseString(s)
            sunrise_el = parsed.getElementsByTagName(
                "morning")[0].getElementsByTagName(
                "sunrise")[0].firstChild.nodeValue
            sunset_el  = parsed.getElementsByTagName(
                "evening")[0].getElementsByTagName(
                "sunset")[0].firstChild.nodeValue
            timezone_el = parsed.getElementsByTagName(
                "timezone")[0].firstChild.nodeValue
            dst_el = parsed.getElementsByTagName(
                "dst")[0].firstChild.nodeValue
            sunrise = datetime.datetime.strptime(sunrise_el, '%H:%M:%S')
            sunset = datetime.datetime.strptime(sunset_el, '%H:%M:%S')
            if dst_el == '1':
                sunrise = datetime.datetime.combine(sunrise.date(),
                    sunrise.time()) - datetime.timedelta(hours=1)
                sunset = datetime.datetime.combine(sunset.date(),
                    sunset.time()) - datetime.timedelta(hours=1)
            return (sunrise.time(), sunset.time())

        except urllib2.URLError:
            self.log.error("GoogleWeather: error reaching url '%s'" % url)

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
        return (None, None)
