#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'YahooWeather',
    ]

import traceback
import datetime
import locale
import pywapi
import re
import urllib2
from urllib import urlencode
from xml.dom.minidom import parseString

from indicator_weather_service import *
from indicator_weather_service.providers import WeatherProvider
from indicator_weather_service.weather import WeatherField, WeatherSection
from indicator_weather_service.units import MetricSystem, WindUnit

import gettext
from gettext import gettext as _
from gettext import ngettext as __
gettext.textdomain(GETTEXT_DOMAIN)

class YahooWeather(WeatherProvider):
    """
    WeatherProvider subclass for Yahoo weather data source

    http://developer.yahoo.com/weather/
    """
    NAME = "Yahoo"

    #Available conditions by yahoo condition code
    #Format: condition code: (day icon, night icon, is a severe weather condition, localized condition name)
    CONDITIONS = {
        '0' : ("weather-storm",             "weather-storm",            True,  _("Tornado")),
        '1' : ("weather-storm",             "weather-storm",            True,  _("Tropical storm")),
        '2' : ("weather-storm",             "weather-storm",            True,  _("Hurricane")),
        '3' : ("weather-storm",             "weather-storm",            True,  _("Severe thunderstorms")),
        '4' : ("weather-storm",             "weather-storm",            True,  _("Thunderstorms")),
        '5' : ("weather-snow",              "weather-snow",             False, _("Mixed rain and snow")),
        # Use Americal meaning of sleet - see http://en.wikipedia.org/wiki/Sleet
        '6' : ("weather-showers",           "weather-showers",          False, _("Mixed rain and sleet")),
        '7' : ("weather-snow",              "weather-snow",             False, _("Mixed snow and sleet")),
        '8' : ("weather-showers",           "weather-showers",          False, _("Freezing drizzle")),
        '9' : ("weather-showers",           "weather-showers",          False, _("Drizzle")),
        '10': ("weather-snow",              "weather-snow",             False, _("Freezing rain")),
        '11': ("weather-showers",           "weather-showers",          False, _("Showers")),
        '12': ("weather-showers",           "weather-showers",          False, _("Showers")),
        '13': ("weather-snow",              "weather-snow",             False, _("Snow flurries")),
        '14': ("weather-snow",              "weather-snow",             False, _("Light snow showers")),
        '15': ("weather-snow",              "weather-snow",             False, _("Blowing snow")),
        '16': ("weather-snow",              "weather-snow",             False, _("Snow")),
        '17': ("weather-showers",           "weather-showers",          False, _("Hail")),
        '18': ("weather-snow",              "weather-snow",             False, _("Sleet")),
        '19': ("weather-fog",               "weather-fog",              False, _("Dust")),
        '20': ("weather-fog",               "weather-fog",              False, _("Foggy")),
        '21': ("weather-fog",               "weather-fog",              False, _("Haze")),
        '22': ("weather-fog",               "weather-fog",              False, _("Smoky")),
        '23': ("weather-clear",             "weather-clear-night",      False, _("Blustery")),
        '24': ("weather-clear",             "weather-clear-night",      False, _("Windy")),
        '25': ("weather-clear",             "weather-clear-night",      False, _("Cold")),
        '26': ("weather-clouds",            "weather-clouds-night",     False, _("Cloudy")),
        '27': ("weather-clouds",            "weather-clouds-night",     False, _("Mostly cloudy")),
        '28': ("weather-clouds",            "weather-clouds-night",     False, _("Mostly cloudy")),
        '29': ("weather-few-clouds",        "weather-few-clouds-night", False, _("Partly cloudy")),
        '30': ("weather-few-clouds",        "weather-few-clouds-night", False, _("Partly cloudy")),
        '31': ("weather-clear",             "weather-clear-night",      False, _("Clear")),
        '32': ("weather-clear",             "weather-clear-night",      False, _("Sunny")),
        '33': ("weather-clear",             "weather-clear-night",      False, _("Fair")),
        '34': ("weather-clear",             "weather-clear-night",      False, _("Fair")),
        '35': ("weather-showers-scattered", "weather-showers-scattered",False, _("Mixed rain and hail")),
        '36': ("weather-clear",             "weather-clear-night",      False, _("Hot")),
        '37': ("weather-storm",             "weather-storm",            True,  _("Isolated thunderstorms")),
        '38': ("weather-storm",             "weather-storm",            True,  _("Scattered thunderstorms")),
        '39': ("weather-storm",             "weather-storm",            True,  _("Scattered thunderstorms")),
        '40': ("weather-showers-scattered", "weather-showers-scattered",False, _("Scattered showers")),
        '41': ("weather-snow",              "weather-snow",             False, _("Heavy snow")),
        '42': ("weather-snow",              "weather-snow",             False, _("Scattered snow showers")),
        '43': ("weather-snow",              "weather-snow",             False, _("Heavy snow")),
        '44': ("weather-few-clouds",        "weather-few-clouds-night", False, _("Partly cloudy")),
        '45': ("weather-storm",             "weather-storm",            True,  _("Thundershowers")),
        '46': ("weather-snow",              "weather-snow",             False, _("Snow showers")),
        '47': ("weather-storm",             "weather-storm",            True,  _("Isolated thundershowers")),
        '3200': (None,                      None,                       False, _("Unknown condition"))
    }

    @staticmethod
    def get_location_id(location_code, location_details, log):
        """
        Returns a location id for a specific location
        """
        try:
            # Get old Yahoo id by woeid
            url = "http://weather.yahooapis.com/forecastrss?w=%s" % location_code
            log.debug("YahooWeather: Get Yahoo RSS ID, url %s" % url)
            f = urllib2.urlopen(url)
            s = f.read()
            parsed = parseString(s)
            guid = parsed.getElementsByTagName("guid")
            if len(guid) > 0:
                guid_value = guid[0].firstChild.nodeValue
                p = re.compile('([^_]*)_')
                m = p.match(guid_value)
                if m:
                    return m.group(1)
                else:
                    log.error("YahooWeather: Can't find yahoo id via woeid." +
                        " Full response:\n %s" % guid)
            else:
                log.error("YahooWeather: Can't find guid in yahoo RSS." +
                        " Full response:\n %s" % s)

        except urllib2.URLError:
            log.error("YahooWeather: error reaching url '%s'" % url)

        except Exception, e:
            log.error(e)
            log.debug(traceback.format_exc(e))
        return None

    def __init__(self, location_id, lat, lon, log):
        """
        Initialize
        """
        super(YahooWeather, self).__init__(location_id, lat, lon, log)

    def fetch_data(self):
        """
        Gets fresh weather data
        """
        # get data in english locale, imperial units
        self.__report = pywapi.get_weather_from_yahoo(
            self._location_id, 'imperial')
        #self.log.debug("YahooWeather: report for %s:\n%s" % \
            #(self._location_id, self.__report))
        # parse data
        return self.__parse_data()

    def __parse_data(self):
        """
        Parses the received data and fills in a dict
        """
        data = {
            WeatherSection.INFO : {
                WeatherField.SOURCE : self.NAME,
                WeatherField.METRIC_SYSTEM : MetricSystem.IMPERIAL,
                WeatherField.WIND_UNIT : WindUnit.MPH,
            }
        }

        item = {}

        # store timestamp
        item[WeatherField.TIMESTAMP] = self._get_timestamp()

        # get sunrise/sunset times
        self._night = False
        if 'astronomy' not in self.__report:
            self.log.warning("YahooWeather: Missing section: astronomy")
        else:
            section = self.__report['astronomy']
            locale.setlocale(locale.LC_ALL, 'C')
            item[WeatherField.SUNRISE] = self._extract_time(section,
                'sunrise', "%I:%M %p")
            item[WeatherField.SUNSET] = self._extract_time(section,
                'sunset', "%I:%M %p")
            locale.resetlocale()

            if item[WeatherField.SUNRISE] is not None:
                # calculate whether it is night or day
                now = datetime.datetime.now().time()
                if now < item[WeatherField.SUNRISE] or \
                        now > item[WeatherField.SUNSET]:
                    self._night = True
                else:
                    self._night = False

        # get condition text, icon, severe
        if 'condition' not in self.__report:
            self.log.warning("YahooWeather: Missing section: condition")
        else:
            section = self.__report['condition']
            code = self._extract_str(section, 'code')
            if code in self.CONDITIONS:
                cond = self.CONDITIONS[code]
                item[WeatherField.CONDITION] = cond[3]
                item[WeatherField.ICON] = cond[1] if self._night else cond[0]
                item[WeatherField.SEVERE] = cond[2]

            # get temp
            item[WeatherField.TEMPERATURE] = self._extract_float(
                section, 'temp')

        # get wind
        if 'wind' not in self.__report:
            self.log.warning("YahooWeather: Missing section: wind")
        else:
            section = self.__report['wind']
            item[WeatherField.WIND_SPEED] = self._extract_float(
                section, 'speed')
            item[WeatherField.WIND_DIRECTION] = \
                WindUnit.convert_degrees_to_direction(
                    self._extract_int(section, 'direction'))

        # get atmosphere
        if 'atmosphere' not in self.__report:
            self.log.warning("YahooWeather: Missing section: atmosphere")
        else:
            section = self.__report['atmosphere']
            item[WeatherField.HUMIDITY] = self._extract_float(
                section, 'humidity')
            item[WeatherField.VISIBILITY] = self._extract_float(
                section, 'visibility')
            item[WeatherField.PRESSURE] = self._extract_float(
                section, 'pressure')
            item[WeatherField.PRESSURE_STATUS] = self._extract_int(
                section, 'rising')

        # store current conditions
        data[WeatherSection.CURRENT] = item

        # get forecast
        if 'forecasts' not in self.__report:
            self.log.warning("YahooWeather: Missing section: forecasts")
        else:
            for index, section in enumerate(self.__report['forecasts']):
                item = {}
                locale.setlocale(locale.LC_ALL, 'C')
                item[WeatherField.FORECAST_DAY] = self._extract_date(
                    section, 'date', "%d %b %Y")
                locale.resetlocale()

                code = self._extract_str(section, 'code')
                if code in self.CONDITIONS:
                    cond = self.CONDITIONS[code]
                    item[WeatherField.ICON] = cond[0]
                    item[WeatherField.CONDITION] = cond[3]
                    item[WeatherField.SEVERE] = cond[2]

                item[WeatherField.TEMP_LOW_HIGH] = (
                    self._extract_float(section, 'low'),
                    self._extract_float(section, 'high'))

                data["forecast%d" % index] = item

        return data
