#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'Server',
    ]

from indicator_weather_service import *

# We need to tell gi.repository where to find our homemade Indicator Typelib
from gi.repository import GIRepository
GIRepository.Repository.prepend_search_path(get_data_file("gir"))
from gi.repository import Indicator

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import glib
import gtk
import traceback

from .indicator import WeatherIndicator

class Server(object):

    def __init__(self, log, keep_alive=False):
        self.log=log
        # need to set dbus main loop before connection to dbus signals
        DBusGMainLoop(set_as_default=True)
        # get session bus
        self.bus=dbus.SessionBus()
        self.keep_alive=keep_alive
        self.indicator_service=None
        self.indicator_service_manager=None
        self.indicator=None
        self.running=False
        self.started=False

    def start(self):
        try:
            # if the name is registered,
            # it means that another instance is running
            if (self.bus.name_has_owner(DBUS_SERVICE_NAME)):
                self.log.info("Server: Service already running. Exiting")
                return

            # initialize the manager
            self.log.info("Server: Initializing Indicator Weather")
            self.indicator = WeatherIndicator(self.log)

            # watch our service name for changes
            self.bus.watch_name_owner(DBUS_SERVICE_NAME,
                self.on_watch_name_owner)

            self.log.info("Server: Initializing Indicator.Service: " + \
                "'%s' version %.0f" % (DBUS_SERVICE_NAME,
                                       DBUS_SERVICE_VERSION))
            # init an indicator service
            self.indicator_service = Indicator.Service.new_version(
                DBUS_SERVICE_NAME, DBUS_SERVICE_VERSION)
            # the shutdown signal will end our life!
            self.indicator_service.connect("shutdown", self.stop)

            if (self.keep_alive):
                # if the service starts before the client
                # we need a ServiceManager running to keep it alive
                self.log.info("Server: Initializing Indicator.ServiceManager")
                self.indicator_service_manager = \
                    Indicator.ServiceManager.new_version(
                        DBUS_SERVICE_NAME, DBUS_SERVICE_VERSION)
                self.indicator_service_manager.connect(
                    "connection-change", self.on_connection_change)

            # Set http proxy support
            ProxyMonitor.monitor_proxy(self.log)
            # Use date-time format as in indicator-datetime
            TimeFormatter.monitor_indicator_datetime(self.log)

            # all ok, enter loop
            self.run()

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            self.stop()

    def stop(self, indicator_service=None):
        self.log.info("Server: Shutting down")
        if self.running:
            gtk.main_quit()
        if self.indicator is not None:
            self.indicator.destroy()
        self.indicator=None
        self.indicator_service=None
        self.indicator_service_manager=None
        self.running=False
        self.started=False

    def run(self):
        self.log.debug("Server: Entering main loop")
        self.running=True
        glib.threads_init()
        gtk.gdk.threads_init()
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()

    def on_name_appeared(self):
        self.log.info("Server: Starting Indicator Weather")
        try:
            self.indicator.start(self.bus)
            self.started=True

        except Exception, e:
            self.log.error(e)
            self.log.debug(traceback.format_exc(e))
            self.stop()
        return False

    def on_name_vanished(self):
        self.log.info("Server: Lost service name '%s'" % \
            DBUS_SERVICE_NAME)
        self.stop()
        return False

    def on_watch_name_owner(self, conn_name):
        self.log.debug("Server: on_watch_name_owner: '%s'" % conn_name)
        # get out of the dbus loop and schedule the actions
        #if not conn_name and self.started:
            #glib.timeout_add(100, self.on_name_vanished)

        if conn_name and not self.started:
            glib.timeout_add(100, self.on_name_appeared)

    def on_connection_change(self, indicator_service_manager, connected):
        self.log.debug("Indicator.ServiceManager connected: %s" % \
            connected);
