#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


__all__ = [
    'Notification'
    'IndicatorDisplay'
    'Settings',
    ]

import traceback
import types
import glib
import couchdb
from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record as CouchRecord
import dbus

from indicator_weather_service import *
from .weather import WeatherField, WeatherSource
from .units import MetricSystem, WindUnit


class Notification:
    """
    Class for available notification methods
    """
    # name constants - setting value
    NONE = 'N'
    ALL = 'A'
    ONLY_SEVERE = 'O'

    # info dict
    # INFO_WIDGET : widget in preferences dialog
    INFO = {
        NONE : {
            INFO_WIDGET : 'nonotif',
        },
        ALL : {
            INFO_WIDGET : 'notifall',
        },
        ONLY_SEVERE : {
            INFO_WIDGET : 'notifsevere',
        },
    }


class IndicatorDisplay:
    """
    Class for available indicator display modes
    """
    # name constants - setting value (we get these from Vala client)
    ICON = DISPLAY_MODE_ICON
    LABEL = DISPLAY_MODE_LABEL
    BOTH = DISPLAY_MODE_BOTH
    NONE = DISPLAY_MODE_NONE

    # info dict
    # INFO_WIDGET : widget in preferences dialog
    INFO = {
        ICON : {
            INFO_WIDGET : 'display_icon',
        },
        LABEL : {
            INFO_WIDGET : 'display_label',
        },
        BOTH : {
            INFO_WIDGET : 'display_both',
        },
        NONE : {
        },
    }


class Settings(object):
    """
    Class for the available application settings.
    It maintains a local dict and a settings backend (couchdb).
    Upon setting a value it automatically saves it to settings backend.
    It can be accessed as an object and as a dict:
     settings.foo
     settings[foo]
     settings.foo=bar
     settings[foo]=bar
    """

    # setting name constants
    INDICATOR_DISPLAY       = 'indicator_display'
    NOTIFICATIONS           = 'notifications'
    GLOBAL_NOTIFICATIONS    = 'global_notifications'
    WEATHER_SOURCE          = 'weather_source'
    REFRESH_RATE            = 'refresh_rate'
    METRIC_SYSTEM           = 'metric_system'
    WIND_UNIT               = 'wind_unit'
    SHOW_FORECAST           = 'show_forecast'
    SHOW_LOCATIONS          = 'show_locations'
    CONDITION_FIELDS        = 'condition_fields'
    PLACECHOSEN             = 'placechosen'
    PLACES                  = 'places'

    # info dict
    # INFO_TYPE : Python type
    # INFO_DEFAULT : default value
    # INFO_VALUES : allowed values
    # INFO_CLASS : associated class
    # INFO_WIDGET : widget in preferences dialog
    INFO = {
        INDICATOR_DISPLAY : {
            INFO_TYPE : types.IntType,
            INFO_DEFAULT : IndicatorDisplay.ICON,
            INFO_VALUES : IndicatorDisplay.INFO.keys(),
            INFO_CLASS : IndicatorDisplay,
        },
        NOTIFICATIONS : {
            INFO_TYPE : types.StringType,
            INFO_DEFAULT : Notification.NONE,
            INFO_VALUES : Notification.INFO.keys(),
            INFO_CLASS : Notification,
        },
        GLOBAL_NOTIFICATIONS : {
            INFO_TYPE : types.BooleanType,
            INFO_DEFAULT : False,
            INFO_WIDGET: 'global_notifications'
        },
        WEATHER_SOURCE : {
            INFO_TYPE : types.StringType,
            INFO_DEFAULT : WeatherSource.YAHOO,
            INFO_VALUES : WeatherSource.INFO.keys(),
            INFO_CLASS : WeatherSource,
        },
        REFRESH_RATE : {
            INFO_TYPE : types.IntType,
            INFO_DEFAULT : 15,
            INFO_VALUES : range(1, 60),
            INFO_WIDGET: 'refreshrate_adjustment'
        },
        METRIC_SYSTEM : {
            INFO_TYPE : types.StringType,
            INFO_DEFAULT : MetricSystem.SI,
            INFO_VALUES : MetricSystem.INFO.keys(),
            INFO_CLASS : MetricSystem,
        },
        WIND_UNIT : {
            INFO_TYPE : types.StringType,
            INFO_DEFAULT : WindUnit.MPH,
            INFO_VALUES : WindUnit.INFO.keys(),
            INFO_CLASS : WindUnit,
        },
        SHOW_FORECAST : {
            INFO_TYPE : types.BooleanType,
            INFO_DEFAULT : True,
            INFO_WIDGET: 'show_forecast'
        },
        SHOW_LOCATIONS : {
            INFO_TYPE : types.BooleanType,
            INFO_DEFAULT : False,
            INFO_WIDGET: 'show_locations'
        },
        CONDITION_FIELDS : {
            INFO_TYPE : types.ListType,
            INFO_DEFAULT : [
                WeatherField.HEAT_INDEX,
                WeatherField.HUMIDITY,
                WeatherField.WIND,
                WeatherField.SUN,
            ],
        },
        PLACECHOSEN : {
            INFO_TYPE : types.IntType,
            INFO_DEFAULT : 0,
            INFO_VALUES : range(0, 100)
        },
        PLACES : {
            INFO_TYPE : types.ListType,
            INFO_DEFAULT : None,
        },
    }

    def __init__(self, log):
        """
        Initializes the object and the settings backend
        """
        self.log = log
        self.changed = list()
        # initialize settings to default values
        self.__dict__.update(map(
            lambda x: (x, Settings.INFO[x].get(INFO_DEFAULT)),
            Settings.INFO.iterkeys()))
        # initialize CouchDb backend
        self.__backend = CouchBackend('weatherindicatorcs', self.log)
        # dict with the loaded location details (key is location code)
        self.__location_details = {}
        # load settings
        for key in Settings.INFO:
            self.__read(key)

    def __del__(self):
        """
        Deletes the object and compacts the settings backend
        """
        if self.__backend is not None:
            try:
                # Compact the settings DB
                #
                # There is a bug in python client v0.6, db.compact raises error
                # http://code.google.com/p/couchdb-python/issues/detail?id=141
                #
                # It seems that the client should always set the
                # 'Content-Type': 'application/json' header
                # So we bypass that compact command and execute a post directly
                # until the python client gets fixed
                headers = {
                    'Content-Type': 'application/json'
                }
                self.__backend.db.db.resource.post('_compact', headers=headers)

            except Exception, e:
                log.debug(e)

    def __repr__(self):
        """
        Returns a dict representation
        """
        return {k: self.__dict__[k]
            for k in Settings.INFO.iterkeys()}.__repr__()

    def __iter__(self):
        """
        Returns a dict iterator
        """
        return {k: self.__dict__[k]
            for k in Settings.INFO.iterkeys()}.__iter__()

    def __getitem__(self, key):
        """
        Dict access, returns a setting
        """
        return self.__dict__[key]

    def __setitem__(self, key, value):
        """
        Dict access, sets a setting
        """
        self.__setattr__(key, value)

    def __setattr__(self, attr, value):
        """
        Sets an attributte and saves it to the settings backend
        """
        if attr in Settings.INFO:
            curr_value = self.__dict__[attr]
            if curr_value != value:
                # special handling for locations
                if attr == Settings.PLACES:
                    value = self.__validate_places(value)
                self.changed.append(attr)
                self.__dict__[attr] = value
                self.__backend.set_value(attr, value)
        else:
            self.__dict__[attr] = value

    def __read(self, key):
        """
        Reads a setting from settings backend and parses it
        """
        value = self.__backend.get_value(key)
        self.log.debug("Settings: read '%s'='%s'" % (key, value))
        valid = False
        value_type = Settings.INFO[key].get(INFO_TYPE, types.StringType)
        allowed_values = Settings.INFO[key].get(INFO_VALUES)
        try:
            if isinstance(value, types.StringTypes):
                if value_type is types.BooleanType:
                    value = True if value == "True" else False
                elif value_type is types.IntType:
                    value = int(value)
                elif value_type in (types.ListType, types.DictType):
                    value = eval(value)
                if isinstance(value, value_type) and \
                        (allowed_values is None or value in allowed_values):
                    valid = True

        except Exception, e:
            self.log.error(e)
            #self.log.debug(traceback.format_exc(e))

        # get current value
        curr_value = self.__dict__[key]

        # special handling for locations
        if key == Settings.PLACES:
            value = self.__validate_places(value)

        if valid:
            if curr_value != value:
                self.__dict__[key] = value
        else:
            self.log.debug("Settings: could not parse " +
                "%s, setting to %s" % (key, curr_value))
            self.__backend.set_value(key, curr_value)

    def __validate_places(self, value):
        """
        Validates a places list
        """
        valid = True
        if value is not None:
            for item in value:
                if not isinstance(item, types.ListType) or len(item)!=2:
                    log.debug("Settings: could not validate places")
                    value = None
                    self.__backend.set_value(Settings.PLACES, None)
                    break

        if value is not None:
            failed = []
            for place in value:
                location_code = place[0]
                ld = self.get_location_details(location_code)
                if ld is None:
                    self.log.debug("Settings: could not parse " +
                        "location details for %s" % location_code)
                    failed.append(place)
            for place in failed:
                value.remove(place)
            if len(failed)>0:
                self.__backend.set_value(Settings.PLACES, value)
        return value

    def get_location_details(self, location_code):
        """
        Returns the location details from cache or settings backend
        """
        if location_code not in self.__location_details:
            try:
                ld = eval(self.__backend.get_location_details(location_code))
                if isinstance(ld, types.DictType):
                    self.__location_details[location_code] = ld
            except:
                return None
        return self.__location_details[location_code]

    def save_location_details(self, location_code, location_details):
        """
        Saves the location details to cache and settings backend
        """
        self.__location_details[location_code] = location_details
        self.__backend.save_location_details(location_code, location_details)

    def get_location_condition(self, location_code):
        """
        Returns the last known condition for a location
        """
        return self.__location_details[location_code].get('condition')

    def save_location_condition(self, location_code, condition):
        """
        Saves the last known condition for a location
        """
        self.__location_details[location_code]['condition'] = condition
        self.__backend.save_location_details(location_code,
            self.__location_details[location_code])

    def get_weather(self, location_code):
        """
        Returns weather data for a location from settings backend
        """
        return self.__backend.get_weather(location_code)

    def save_weather(self, location_code, weather_data):
        """
        Saves weather data for a location to settings backend
        """
        return self.__backend.save_weather(location_code, weather_data)


class CouchBackend(object):
    """
    Class to read/write settings to CouchDb
    """
    db = None

    def __init__(self, store_name, log):
        self.log=log
        self.prepare_settings_store(store_name)

    # Open the DB and prepare views
    #TODO: Create view when package is installed?
    def prepare_settings_store(self, store_name):
        try:
            self.log.debug("CouchBackend: preparing settings store: %s" % store_name)
            self.db = CouchDatabase(store_name, create = True)

            # Settings view
            self.settings_design_doc = "settings"
            self.settings_view = "function(doc) {if (doc.record_type == '%s') {emit(doc.name, doc);}}" % self.settings_design_doc
            self.db.add_view("get_setting", self.settings_view, None, self.settings_design_doc)

            # Cached weather view
            self.weather_design_doc = "weather"
            self.weather_view = "function(doc) {if (doc.record_type == '%s') {emit(doc.location, doc)} ;}" % self.weather_design_doc
            self.db.add_view("get_weather", self.weather_view, None, self.weather_design_doc)

            # Location info view
            self.location_design_doc = "location"
            self.location_view = "function(doc) {if (doc.record_type == '%s') {emit(doc.location, doc)} ;}" % self.location_design_doc
            self.db.add_view("get_location", self.location_view, None, self.location_design_doc)

        except dbus.DBusException as e:
            self.log.debug("CouchBackend: DBus exception occurred, retrying in 30 secs")
            if hasattr(self, 'refresh_id'):
                glib.source_remove(self.refresh_id)
            self.refresh_id = glib.timeout_add(10000, self.prepare_settings_store)

    # Get a value of the setting
    def get_value(self, setting, return_id = False):
        self.log.debug("CouchBackend: getting value for %s" % setting)
        view = self.db.execute_view("get_setting", self.settings_design_doc)[setting]
        if hasattr(view, 'rows') and len(view.rows) > 0:
            if return_id:
                return str(view.rows[0].value['_id'])
            else:
                return str(view.rows[0].value['value'])
        else:
            self.log.debug("CouchBackend: can't find value for %s" % setting)
            return None

    # Set a setting value
    def set_value(self, setting, value):
        self.log.debug("CouchBackend: setting '%s'='%s'" % (setting, value))
        self.record = {
            "name" : setting,
            "value": value
        }
        #Update previously created document, if exists
        old_doc_id = self.get_value(setting, return_id=True)
        if old_doc_id != None:
            self.log.debug("CouchBackend: setting '%s' was updated" % setting)
            self.db.update_fields(old_doc_id, self.record)
        else:
            self.log.debug("CouchBackend: setting '%s' was created" % setting)
            self.db.put_record(CouchRecord(self.record, self.settings_design_doc))

    # Get cached weather by location code.
    # If return_id is True, only document id is returned, otherwise - full weather data
    def get_weather(self, location_code, return_id = False):
        self.log.debug("CouchBackend: getting cached weather for %s" % location_code)
        view = self.db.execute_view("get_weather", self.weather_design_doc)[location_code]
        if hasattr(view, 'rows') and len(view.rows) > 0:
            if return_id:
                return str(view.rows[0].value['_id'])
            else:
                return str(view.rows[0].value['data'])
        else:
            self.log.debug("CouchBackend: can't find cached weather for %s" % location_code)
            return None

    # Save weather info in cache for specific location
    def save_weather(self, location_code, weather_data):
        self.log.debug("CouchBackend: Saving cached weather data")
        self.record = {
          "location" : location_code,
          "data"     : weather_data
        }
        #Update previously created document, if exists
        old_doc_id = self.get_weather(location_code, return_id=True)
        if old_doc_id != None:
            self.db.update_fields(old_doc_id, self.record)
        else:
            self.db.put_record(CouchRecord(self.record, self.weather_design_doc))

    # Get location details by location code
    # If return_id is True, only document id is returned, otherwise - full location data
    def get_location_details(self, location_code, return_id = False):
        self.log.debug("CouchBackend: getting location details for %s" % location_code)
        view = self.db.execute_view("get_location", self.location_design_doc)[location_code]
        if hasattr(view, 'rows') and len(view.rows) > 0:
            if return_id:
                return str(view.rows[0].value['_id'])
            else:
                return str(view.rows[0].value['data'])
        else:
            self.log.debug("CouchBackend: can't find location details for %s" % location_code)
            return None

    # Save location details
    def save_location_details(self, location_code, location_details):
        self.log.debug("CouchBackend: Saving location details data")
        self.record = {
          "location" : location_code,
          "data"      : {
            "label"    : location_details['label'],
            "full name": location_details['full name'],
            "latitude" : location_details['latitude'],
            "longitude": location_details['longitude'],
          }
        }
        for key in ("google id", "yahoo id", "noaa id", "condition"):
            if key in location_details:
                self.record["data"][key]=location_details[key]

        # Update previously created element, if exists
        old_doc_id = self.get_location_details(location_code, return_id=True)
        if old_doc_id != None:
            self.db.update_fields(old_doc_id, self.record)
            self.log.debug("CouchBackend: Updating location details data")
        else:
            self.db.put_record(CouchRecord(self.record, self.location_design_doc))
            self.log.debug("CouchBackend: Adding new location details data")
