#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'MetricSystem',
    'WindUnit',
    ]

from indicator_weather_service import *

import gettext
from gettext import gettext as _
from gettext import ngettext as __
gettext.textdomain(GETTEXT_DOMAIN)


class MetricSystem:
    """
    Class with available metric systems and conversions
    """
    # name constants - setting value
    SI = 'S'
    IMPERIAL = 'I'

    # info dict
    # INFO_WIDGET : widget in preferences dialog
    # INFO_LABEL : temperature suffix label
    INFO = {
        SI : {
            INFO_WIDGET : 'si',
        },
        IMPERIAL : {
            INFO_WIDGET : 'imperial',
        },
    }

    # barometric pressure status symbols
    PRESSURE_STATUS_SYMBOLS = {
        0 : '',
        1 : '↑',
        2 : '↓',
    }

    # ------------------------------------------------------------
    # Temperature methods
    # ------------------------------------------------------------

    @staticmethod
    def convert_F_to_C(value):
        if value is None:
            return None
        return (float(value) - 32.) * 5. / 9.

    @staticmethod
    def convert_C_to_F(value):
        if value is None:
            return None
        return float(value) * 9. / 5. + 32.

    @staticmethod
    def convert_temperature(from_system, to_system, value):
        if from_system == to_system:
            return value
        elif from_system == MetricSystem.SI and to_system == MetricSystem.IMPERIAL:
            return MetricSystem.convert_C_to_F(value)
        elif from_system == MetricSystem.IMPERIAL and to_system == MetricSystem.SI:
            return MetricSystem.convert_F_to_C(value)
        else:
            return value

    @staticmethod
    def get_temperature_unit(unit, value):
        """
        Gets the temperature unit
        """
        if value is None:
            return ""

        elif unit == MetricSystem.SI:
            return "°C"

        elif unit == MetricSystem.IMPERIAL:
            return "°F"

        else:
            return ""

    # ------------------------------------------------------------
    # Distance methods
    # ------------------------------------------------------------

    @staticmethod
    def convert_km_to_miles(value):
        if value is None:
            return None
        return float(value) / 1.609344

    @staticmethod
    def convert_miles_to_km(value):
        if value is None:
            return None
        return float(value) * 1.609344

    @staticmethod
    def convert_distance(from_system, to_system, value):
        if from_system == to_system:
            return value
        elif from_system == MetricSystem.SI and to_system == MetricSystem.IMPERIAL:
            return MetricSystem.convert_km_to_miles(value)
        elif from_system == MetricSystem.IMPERIAL and to_system == MetricSystem.SI:
            return MetricSystem.convert_miles_to_km(value)
        else:
            return value

    @staticmethod
    def get_distance_unit(unit, value):
        """
        Gets the translated distance unit
        """
        if value is None:
            return ""

        elif unit == MetricSystem.SI:
            return __("km", "km", value)

        elif unit == MetricSystem.IMPERIAL:
            return __("mi", "mi", value)

        else:
            return ""

    # ------------------------------------------------------------
    # Pressure methods
    # ------------------------------------------------------------

    @staticmethod
    def convert_millibar_to_inch(value):
        if value is None:
            return None
        return float(value) / 33.8637526

    @staticmethod
    def convert_inch_to_millibar(value):
        if value is None:
            return None
        return float(value) * 33.8637526

    @staticmethod
    def convert_pressure(from_system, to_system, value):
        if from_system == to_system:
            return value
        elif from_system == MetricSystem.SI and to_system == MetricSystem.IMPERIAL:
            return MetricSystem.convert_millibar_to_inch(value)
        elif from_system == MetricSystem.IMPERIAL and to_system == MetricSystem.SI:
            return MetricSystem.convert_inch_to_millibar(value)
        else:
            return value

    @staticmethod
    def get_pressure_unit(unit, value):
        """
        Gets the translated pressure unit
        """
        if value is None:
            return ""

        elif unit == MetricSystem.SI:
            return __("mb", "mb", value)

        elif unit == MetricSystem.IMPERIAL:
            return __("in", "in", value)

        else:
            return ""

    @staticmethod
    def convert_pressure_status_to_symbol(status):
        """
        Converts barometric pressure status from int to arrow symbols
        """
        if status is None:
            return ''
        return MetricSystem.PRESSURE_STATUS_SYMBOLS.get(status, '')


class WindUnit:
    """
    Class for available wind units and conversions
    """
    # name constants - setting value
    MPS = 'mps'
    MPH = 'mph'
    KPH = 'kph'
    BEAUFORT = 'beaufort'
    KNOTS = 'knots'

    # info dict
    # INFO_WIDGET : widget in preferences dialog
    INFO = {
        MPS : {
            INFO_WIDGET : "mps",
        },
        MPH : {
            INFO_WIDGET : "mph",
        },
        KPH : {
            INFO_WIDGET : "kph",
        },
        BEAUFORT : {
            INFO_WIDGET : "beaufort",
        },
        KNOTS : {
            INFO_WIDGET : "knots",
        },
    }

    # wind direction symbols
    ARROWS_NORMAL = {
        'N' : '↓', 'NE': '↙',
        'E' : '←', 'SE': '↖',
        'S' : '↑', 'SW': '↗',
        'W' : '→', 'NW': '↘',
    }
    ARROWS_DOUBLE = {
        'N' : '⇓', 'NE': '⇙',
        'E' : '⇐', 'SE': '⇖',
        'S' : '⇑', 'SW': '⇗',
        'W' : '⇒', 'NW': '⇘',
    }

    # ------------------------------------------------------------
    # Wind Speed methods
    # ------------------------------------------------------------

    @staticmethod
    def convert_speed_from_mph(unit, value):
        """
        Converts mph to other units
        """
        if value is None:
            return None

        elif unit == WindUnit.MPH:
            return value

        elif unit == WindUnit.MPS:
            return value * 0.44704

        elif unit == WindUnit.KPH:
            return value * 1.609344

        elif unit == WindUnit.BEAUFORT:
            if value < 1:
                return 0
            elif value < 4:
                return 1
            elif value < 8:
                return 2
            elif value < 13:
                return 3
            elif value < 18:
                return 4
            elif value < 25:
                return 5
            elif value < 27:
                return 6
            elif value < 39:
                return 7
            elif value < 47:
                return 8
            elif value < 89:
                return 9
            elif value < 64:
                return 10
            elif value < 73:
                return 11
            elif value >= 73:
                return 12

        elif unit == WindUnit.KNOTS:
            return value * 0.868976241900648

        else:
            return value

    @staticmethod
    def convert_speed_to_mph(unit, value):
        """
        Converts other units to mph
        """
        if value is None:
            return None

        elif unit == WindUnit.MPH:
            return value

        elif unit == WindUnit.MPS:
            return value / 0.44704

        elif unit == WindUnit.KPH:
            return value / 1.609344

        elif unit == WindUnit.BEAUFORT:
            if value < 1:
                return 0
            elif value == 1:
                return 2
            elif value == 2:
                return 5.5
            elif value == 3:
                return 10
            elif value == 4:
                return 15.5
            elif value == 5:
                return 21.5
            elif value == 6:
                return 28
            elif value == 7:
                return 35
            elif value == 8:
                return 42.5
            elif value == 9:
                return 50.5
            elif value == 10:
                return 59
            elif value == 11:
                return 68
            elif value >= 12:
                return 73

        elif unit == WindUnit.KNOTS:
            return value / 0.868976241900648

        else:
            return value

    @staticmethod
    def convert_speed(from_unit, to_unit, value):
        if from_unit == to_unit:
            return value
        elif from_unit == WindUnit.MPH:
            return WindUnit.convert_speed_from_mph(to_unit, value)
        elif to_unit == WindUnit.MPH:
            return WindUnit.convert_speed_to_mph(from_unit, value)
        else:
            return WindUnit.convert_speed_from_mph(to_unit,
                WindUnit.convert_speed_to_mph(from_unit, value))

    @staticmethod
    def get_speed_unit(unit, value):
        """
        Gets the translated speed unit
        """
        if value is None:
            return ""

        elif unit == WindUnit.MPS:
            return __("m/s", "m/s", value)

        elif unit == WindUnit.MPH:
            return __("mph", "mph", value)

        elif unit == WindUnit.KPH:
            return __("km/h", "km/h", value)

        elif unit == WindUnit.BEAUFORT:
            return ""

        elif unit == WindUnit.KNOTS:
            return __("knot", "knots", value)

        else:
            return ""

    # ------------------------------------------------------------
    # Wind Direction methods
    # ------------------------------------------------------------

    @staticmethod
    def convert_degrees_to_direction(value):
        """
        Converts wind direction from degrees to textual direction
        """
        try:
            degrees = int(value)
        except:
            return ''

        if degrees < 23 or degrees >= 338:
            #Short wind direction - north
            return 'N'
        elif degrees < 68:
            return 'NE'
        elif degrees < 113:
            return 'E'
        elif degrees < 158:
            return 'SE'
        elif degrees < 203:
            return 'S'
        elif degrees < 248:
            return 'SW'
        elif degrees < 293:
            return 'W'
        elif degrees < 338:
            return 'NW'
        return ''

    @staticmethod
    def convert_direction_to_symbol(direction, speed):
        """
        Converts wind direction from text to arrow symbols
        """
        if speed is None or int(speed)==0:
            return ''
        return WindUnit.ARROWS_NORMAL.get(direction, '')
