#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Sebastian MacDonald Sebas310@gmail.com
# Copyright (C) 2010 Mehdi Rejraji mehd36@gmail.com
# Copyright (C) 2011 Vadim Rutkovsky roignac@gmail.com
# Copyright (C) 2011 Panagiotis Skintzos panajotis@gmail.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

__all__ = [
    'Weather',
    'WeatherSection',
    'WeatherField',
    'WeatherSource',
    ]

import traceback
import re
import datetime
import types
import math

from indicator_weather_service import *

import gettext
from gettext import gettext as _
from gettext import ngettext as __
gettext.textdomain(GETTEXT_DOMAIN)

from .units import MetricSystem, WindUnit

class WeatherSource:
    """
    Class for available weather data sources
    """
    # name constants - setting value
    CACHE = 'CACHE'
    GOOGLE = 'G'
    YAHOO = 'Y'

    # info dict
    # INFO_WIDGET : widget in preferences dialog
    # INFO_TYPE : class name (WeatherProvider subclass)
    # INFO_LABEL : location id key name
    INFO = {
        CACHE : {
            INFO_CLASS : 'providers.cached.CachedWeather',
        },
        GOOGLE : {
            INFO_WIDGET : 'google',
            INFO_CLASS : 'providers.google.GoogleWeather',
            INFO_LABEL : 'google id'
        },
        YAHOO : {
            INFO_WIDGET : 'yahoo',
            INFO_CLASS : 'providers.yahoo.YahooWeather',
            INFO_LABEL : 'yahoo id'
        },
    }

    _CLASSES = {}

    @staticmethod
    def load_class(name):
        if name not in WeatherSource._CLASSES:
            clsname = WeatherSource.INFO[name][INFO_CLASS]
            module_name, member_name = clsname.rsplit(".", 1)
            module_name = "indicator_weather_service." + module_name
            module = __import__(module_name, fromlist=[member_name])
            WeatherSource._CLASSES[name] = getattr(module, member_name)
        return WeatherSource._CLASSES[name]


class WeatherSection:
    """
    Class for available weather data sections
    """
    # name constants
    INFO      = "info"          # info section
    CURRENT   = "current"       # current conditions
    FORECAST0 = "forecast0"     # today's forecast
    FORECAST1 = "forecast1"     # forecast day 1
    FORECAST2 = "forecast2"     # forecast day 2
    FORECAST3 = "forecast3"     # forecast day 3

    ALL_SECTIONS = (INFO, CURRENT, FORECAST0, FORECAST1, FORECAST2, FORECAST3, )
    FORECAST_SECTIONS = (FORECAST0, FORECAST1, FORECAST2, FORECAST3, )

class WeatherField:
    """
    Class for available weather data fields

    Fields can be simple (like CONDITION), calculated by others
    (like SUN).

    For each field can be defined:
     - type (Python type)
     - flags which control internal functionality
     - a label for display in menu
     - a handler function, which should return the pure value
     - a formatter function, which handles the formatting

    These functions should be defined in the Weather class.
    """

    # name constants
    TIMESTAMP          = "timestamp"
    CONDITION          = "condition"
    ICON               = "icon"
    SEVERE             = "severe"
    TEMPERATURE        = "temperature"
    TEMPERATURE_SHORT  = "temperature-short"
    HEAT_INDEX         = "heat-index"
    APPARENT_TEMP      = "apparent-temp"
    HUMIDEX            = "humidex"
    DEWPOINT           = "dewpoint"
    HUMIDITY           = "humidity"
    PRESSURE           = "pressure"
    PRESSURE_STATUS    = "pressure-status"
    BAROMETER          = "barometer"
    VISIBILITY         = "visibility"
    WIND               = "wind"
    WIND_DIRECTION     = "wind-dir"
    WIND_SPEED         = "wind-speed"
    SUN                = "sun"
    SUNRISE            = "sunrise"
    SUNSET             = "sunset"
    FORECAST_DAY       = "forecast-day"
    TEMP_LOW_HIGH      = "temp-low-high"
    LOCATION           = "location"
    METRIC_SYSTEM      = "metric_system"
    WIND_UNIT          = "wind_unit"
    SOURCE             = "source"

    # field flags for bitwise operations
    FLAG_INFO       = 1  # field is present in the info section
    FLAG_CURRENT    = 2  # field is present in the current section
    FLAG_FORECAST   = 4  # field is present in the forecast section
    FLAG_CACHED     = 8  # field is saved in cache
    FLAG_REQUIRED   = 16 # field is required, otherwise the whole section is discarded
    FLAG_CONDITION  = 32 # field is presented in preferences condition fields list

    # info dict for field properties
    INFO = {
        # current location
        LOCATION : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_INFO,
        },
        # weather source
        SOURCE : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_INFO | FLAG_CACHED | FLAG_REQUIRED,
        },
        # metric system used in the report
        METRIC_SYSTEM : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_INFO | FLAG_CACHED | FLAG_REQUIRED,
            INFO_VALUES     : MetricSystem.INFO.keys()
        },
        # wind unit used in the report
        WIND_UNIT : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_INFO | FLAG_CACHED | FLAG_REQUIRED,
            INFO_VALUES     : WindUnit.INFO.keys()
        },
        # timestamp of the report
        TIMESTAMP : {
            INFO_FORMATTER  : "_format_timestamp",
            INFO_TYPE       : datetime.datetime,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_REQUIRED,
        },
        # condition text
        CONDITION : {
            INFO_LABEL      : _("Condition"),
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_FORECAST | FLAG_CACHED | \
                              FLAG_REQUIRED,
        },
        # condition icon
        ICON : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_FORECAST | FLAG_CACHED,
        },
        # is severe condition
        SEVERE : {
            INFO_TYPE       : types.BooleanType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_FORECAST | FLAG_CACHED,
        },
        # temperature
        TEMPERATURE : {
            INFO_LABEL      : _("Temperature"),
            INFO_HANDLER    : "_get_temperature",
            INFO_FORMATTER  : "_format_temperature",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_REQUIRED,
        },
        # temperature formatted with no decimals and units (for indicator label)
        TEMPERATURE_SHORT : {
            INFO_LABEL      : _("Temperature"),
            INFO_HANDLER    : "_get_temperature",
            INFO_FORMATTER  : "_format_temperature_short",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT,
        },
        # calculated heat index
        HEAT_INDEX : {
            INFO_LABEL      : _("Feels like"),
            INFO_HANDLER    : "_get_heat_index",
            INFO_FORMATTER  : "_format_temperature",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # calculated apparent temperature
        APPARENT_TEMP : {
            INFO_LABEL      : _("Feels like (AT)"),
            INFO_HANDLER    : "_get_apparent_temperature",
            INFO_FORMATTER  : "_format_temperature",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # calculated humidex
        HUMIDEX : {
            INFO_LABEL      : _("Humidex"),
            INFO_HANDLER    : "_get_humidex",
            INFO_FORMATTER  : "_format_humidex",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # calculated dewpoint
        DEWPOINT : {
            INFO_LABEL      : _("Dewpoint"),
            INFO_HANDLER    : "_get_dewpoint",
            INFO_FORMATTER  : "_format_temperature",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # relative humidity
        HUMIDITY : {
            INFO_LABEL      : _("Humidity"),
            INFO_HANDLER    : "_get_humidity",
            INFO_FORMATTER  : "_format_humidity",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_CONDITION,
        },
        # visibility
        VISIBILITY : {
            INFO_LABEL      : _("Visibility"),
            INFO_HANDLER    : "_get_visibility",
            INFO_FORMATTER  : "_format_visibility",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_CONDITION,
        },
        # barometric pressure value
        PRESSURE : {
            INFO_HANDLER    : "_get_pressure",
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED,
        },
        # status of barometric pressure (steady, rising, falling)
        PRESSURE_STATUS : {
            INFO_HANDLER    : "_get_pressure_status",
            INFO_TYPE       : types.IntType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED,
        },
        # formatted barometric pressure and status
        BAROMETER : {
            INFO_LABEL      : _("Barometer"),
            INFO_HANDLER    : "_get_barometer",
            INFO_FORMATTER  : "_format_barometer",
            INFO_TYPE       : types.TupleType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # formatted wind direction and speed
        WIND : {
            INFO_LABEL      : _("Wind"),
            INFO_HANDLER    : "_get_wind",
            INFO_FORMATTER  : "_format_wind",
            INFO_TYPE       : types.TupleType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # wind speed
        WIND_SPEED : {
            INFO_TYPE       : types.FloatType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED,
        },
        # wind direction
        WIND_DIRECTION : {
            INFO_TYPE       : types.StringType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED,
        },
        # merged sunrise/sunset
        SUN : {
            INFO_LABEL      : _("Sun"),
            INFO_HANDLER    : "_get_sunrise_sunset",
            INFO_FORMATTER  : "_format_time_pair",
            INFO_TYPE       : types.TupleType,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CONDITION,
        },
        # sunrise time
        SUNRISE : {
            INFO_LABEL      : _("Sunrise"),
            INFO_FORMATTER  : "_format_time",
            INFO_TYPE       : datetime.datetime,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_CONDITION,
        },
        # sunset time
        SUNSET : {
            INFO_LABEL      : _("Sunset"),
            INFO_FORMATTER  : "_format_time",
            INFO_TYPE       : datetime.datetime,
            INFO_FLAGS      : FLAG_CURRENT | FLAG_CACHED | FLAG_CONDITION,
        },
        # forecast day
        FORECAST_DAY : {
            INFO_FORMATTER  : "_format_day",
            INFO_TYPE       : datetime.datetime,
            INFO_FLAGS      : FLAG_FORECAST | FLAG_CACHED | FLAG_REQUIRED,
        },
        # low and high temperature tuple
        TEMP_LOW_HIGH : {
            INFO_HANDLER    : "_get_temperature_low_high",
            INFO_FORMATTER  : "_format_temperature_pair",
            INFO_TYPE       : types.TupleType,
            INFO_FLAGS      : FLAG_FORECAST | FLAG_CACHED | FLAG_REQUIRED,
        },
    }

    # fields to display in preference dialog condition fields,
    # with proper order
    ALL_CONDITION_FIELDS = (
        HEAT_INDEX,
        APPARENT_TEMP,
        HUMIDEX,
        BAROMETER,
        HUMIDITY,
        VISIBILITY,
        DEWPOINT,
        WIND,
        SUNRISE,
        SUNSET,
        SUN,
    )

    @staticmethod
    def get_fields_by_flag(flag):
        """
        Returns the fields that match a flag condition
        """
        return [ x for x in WeatherField.INFO
            if WeatherField.INFO[x].get(INFO_FLAGS, 0) & flag ]


class Weather(object):
    """
    Data object to parse weather data with unit conversion.

    The available data fields are organized in sections, according to
    WeatherSection class

    The fields are accesed through the "get_field_value" method.
    Their labels are accesed through the "get_field_label" method.
    Once accessed, a field formatted value is stored in a dict,
    for quicker future access.
    """

    # cached data for validation
    SECTION_INFO = None

    # default precision to use for float formatting
    TEMPERATURE_PRECISION = 0   # we could put this in settings
    HUMIDITY_PRECISION = 0
    WINDSPEED_PRECISION = -2    # calculated automatically
    DISTANCE_PRECISION = -2     # calculated automatically
    PRESSURE_PRECISION = -2     # calculated automatically

    @staticmethod
    def prepare_section_info():
        print "PREPARING Weather.SECTION_INFO"
        Weather.SECTION_INFO = dict()
        for section_key, section_flag in (
                (WeatherSection.INFO, WeatherField.FLAG_INFO),
                (WeatherSection.CURRENT, WeatherField.FLAG_CURRENT) ):

            Weather.SECTION_INFO[section_key] = (
                # required section?
                True,
                # required fields
                WeatherField.get_fields_by_flag(section_flag &
                                                WeatherField.FLAG_REQUIRED),
                # all fields
                WeatherField.get_fields_by_flag(section_flag),
            )

        section_flag = WeatherField.FLAG_FORECAST
        required_fields = WeatherField.get_fields_by_flag(section_flag & \
            WeatherField.FLAG_REQUIRED)
        all_fields = WeatherField.get_fields_by_flag(section_flag)
        for section_key in WeatherSection.FORECAST_SECTIONS:
            Weather.SECTION_INFO[section_key] = (
                # required section?
                False,
                # required fields
                required_fields,
                # all fields
                all_fields,
            )

    def __init__(self, location_code, location_details, settings, log):
        """
        Inits variables
        """
        self.log = log
        self.__location_code = location_code
        self.__location_details = location_details
        self.__metric_system = None
        self.__wind_unit = None
        self.__weather_source = None
        self.__location_id = None
        self.__settings = settings
        self.__provider = None

        self.__report_metric_system = MetricSystem.IMPERIAL
        self.__report_wind_unit = WindUnit.MPH

        if Weather.SECTION_INFO is None:
            Weather.prepare_section_info()

        # Raw data dict
        # contains the data returned from the providers in US units
        self.__raw = {}
        # Formatted data dict
        # contains formatted values in current units
        self.__formatted = {}

        # load cached weather for location
        cls = WeatherSource.load_class(WeatherSource.CACHE)
        self.__provider = cls(location_code, settings, self.log)
        self.update()

    def set_details(self, location_details,
                    metric_system, wind_unit, weather_source):
        needs_new_data = False
        needs_refresh = False
        if self.__location_details != location_details:
            self.__location_details = location_details
            needs_new_data = True

        weather_source, location_id = self.__resolve_weather_source(
            weather_source)
        if self.__weather_source != weather_source:
            self.__weather_source = weather_source
            needs_new_data = True
        if self.__location_id != location_id:
            self.__location_id = location_id
            needs_new_data = True

        if self.__metric_system != metric_system:
            self.__metric_system = metric_system
            needs_refresh = True

        if self.__wind_unit != wind_unit:
            self.__wind_unit = wind_unit
            needs_refresh = True

        # reset formatted storage
        if needs_refresh:
            self.__formatted = {}
            for key in self.__raw:
                self.__formatted[key] = {}

        if needs_new_data and self.__weather_source is not None:
            cls = WeatherSource.load_class(self.__weather_source)
            self.log.debug(("Weather: loading new provider '%s'" +
                "for [%s %s]") % (cls.NAME,
                self.__location_code, self.__location_details['label']))
            self.__provider = cls(self.__location_id,
                self.__location_details['latitude'],
                self.__location_details['longitude'],
                self.log)

        return needs_new_data

    def update(self):
        if self.__provider is not None:
            data = None
            try:
                data = self.__provider.fetch_data()
            except Exception, e:
                self.log.error(e)
                self.log.debug(traceback.format_exc(e))

            self.log.debug(("Weather: received report from %s " +
                "for [%s %s]:\n%s") % (self.__provider.NAME,
                self.__location_code, self.__location_details['label'],
                data))

            # validate data
            data = self.__validate_data(data)
            if data is not None:
                # store new data
                self.__raw = data
                self.__formatted = {}
                for key in self.__raw:
                    self.__formatted[key] = dict()
                # save weather in CouchDb
                # TODO: Implement this
                #self.__settings.save_weather(self.__location_code, data)
                return True
        return False

    def __validate_data(self, data):
        if data is None:
            return None

        new = dict()
        for section_key in Weather.SECTION_INFO:
            section_required, required_fields, all_fields = \
                Weather.SECTION_INFO[section_key]

            # check required section
            if section_key not in data:
                if section_required:
                    self.log.warning("Weather: data validation failed: " +
                        "missing required section '%s'" % section_key)
                    return None
                else:
                    continue

            section = data[section_key]

            # check the required fields belonging in this section
            for field in required_fields:
                if field not in section:
                    self.log.warning("Weather: data validation failed: " +
                        "missing required field '%s' from section '%s'" % \
                            (field, section_key))
                    if section_required:
                        return None
                    else:
                        break

            # grab all fields belonging in this section
            else:
                fields = [ x for x in section if x in all_fields ]
                new[section_key] = dict(map(lambda x: (x, section[x]), fields))


        # validate some fields
        sinfo = new[WeatherSection.INFO]
        for key in (WeatherField.METRIC_SYSTEM, WeatherField.WIND_UNIT):
            allowed_values = WeatherField.INFO[key].get(INFO_VALUES)
            if allowed_values is not None and sinfo[key] not in allowed_values:
                self.log.warning("Weather: data validation failed: " +
                    "invalid value '%s' for field '%s'" % (sinfo[key], key))
                return None

        self.__report_metric_system = sinfo[WeatherField.METRIC_SYSTEM]
        self.__report_wind_unit = sinfo[WeatherField.WIND_UNIT]
        return new

    def __resolve_weather_source(self, source):
        # gather existing source keys
        valid_source = None
        loc_ids = {}
        for key in WeatherSource.INFO:
            if INFO_LABEL in WeatherSource.INFO[key]:
                label = WeatherSource.INFO[key][INFO_LABEL]
                if label in self.__location_details:
                    loc_ids[key] = self.__location_details[label]

        # try with the default source
        if source in loc_ids:
            valid_source = source
            self.log.debug(("Weather: default provider '%s' " +
                "chosen for [%s %s]") % (valid_source,
                self.__location_code, self.__location_details['label']))

        # try with the first alternative
        elif len(loc_ids.keys()):
            valid_source = loc_ids.keys()[0]
            self.log.debug(("Weather: non default provider '%s' " +
                "chosen for [%s %s]") % (valid_source,
                self.__location_code, self.__location_details['label']))

        if valid_source is None:
            self.log.debug(("Weather: no valid provider can be " +
                "chosen for [%s %s]") % (self.__location_code,
                self.__location_details['label']))
            return (None, None)

        return (valid_source, loc_ids[valid_source])

    def section_exists(self, section):
        """
        Returns True if specified section exists in the report
        """
        return self.__formatted is not None and section in self.__formatted

    def get_sections(self):
        """
        Returns a list of present sections in the report
        """
        return self.__raw.keys() if self.__raw is not None else []

    @staticmethod
    def get_field_label(name):
        """
        Returns the label for a field
        """
        if name not in WeatherField.INFO:
            print ("Weather: unknown field: %s" % name)
            return name
        return WeatherField.INFO[name].get(INFO_LABEL, "<%s>" % name)

    def get_field_value(self, name, section = WeatherSection.CURRENT):
        """
        Returns the formatted value of a field or MISSING_VALUE
        """
        value = MISSING_VALUE

        # location label
        if name == WeatherField.LOCATION:
            return self.__location_details['label']

        # check section name
        elif section not in self.__formatted:
            self.log.warning("Weather: unknown section: %s" % section)
            return value

        # check if it already exists in formatted dict
        elif name in self.__formatted[section]:
            value = self.__formatted[section][name]

        # check field name
        elif name not in WeatherField.INFO:
            self.log.warning("Weather: unknown field: %s" % name)
            return value

        # calculate value
        else:
            # get handler function
            funcname = WeatherField.INFO[name].get(INFO_HANDLER)
            if funcname is None:
                # get value from raw dict
                value = self.__raw[section].get(name)
            else:
                if not hasattr(self, funcname):
                    self.log.warning("Weather: unimplemented method:" +
                        " %s" % funcname)
                else:
                    try:
                        # get value from handler func
                        value = getattr(self, funcname)(section)
                    except Exception, e:
                        self.log.error("Weather: error calculating " +
                            "'%s': %s" % (name, e))
                        self.log.debug(traceback.format_exc(e))

            # check that we have a value
            if value is None:
                value = MISSING_VALUE

            # get formatter function
            else:
                funcname = WeatherField.INFO[name].get(INFO_FORMATTER)
                if funcname is not None:
                    if not hasattr(self, funcname):
                        self.log.warning("Weather: unimplemented method:" +
                            " %s" % funcname)
                    else:
                        try:
                            # get formatted value from formatter func
                            value = getattr(self, funcname)(value)
                            if value is not None:
                                value = value.strip()
                        except Exception, e:
                            value = MISSING_VALUE
                            self.log.error("Weather: error formatting " +
                                "'%s': %s" % (name, e))
                            self.log.debug(traceback.format_exc(e))

            # store formatted value
            self.__formatted[section][name] = value

        return value

    def _get_sunrise_sunset(self, section):
        """
        Returns sunrise and sunset times in a tuple
        """
        sunrise = self.__raw[section].get(WeatherField.SUNRISE)
        sunset = self.__raw[section].get(WeatherField.SUNSET)
        if sunrise is None or sunset is None:
            return None
        return (sunrise, sunset)

    def _get_temperature(self, section, metric_system = None):
        """
        Returns the temperature value in current units or
        in specified metric system
        """
        if metric_system is not None:
            return MetricSystem.convert_temperature(
                self.__report_metric_system, metric_system,
                self.__raw[section].get(WeatherField.TEMPERATURE))
        else:
            return MetricSystem.convert_temperature(
                self.__report_metric_system, self.__metric_system,
                self.__raw[section].get(WeatherField.TEMPERATURE))

    def _get_temperature_low_high(self, section):
        """
        Returns the low/high temperature values in current units
        """
        t =  self.__raw[section].get(WeatherField.TEMP_LOW_HIGH)
        return (
            MetricSystem.convert_temperature(self.__report_metric_system,
                self.__metric_system, t[0]),
            MetricSystem.convert_temperature(self.__report_metric_system,
                self.__metric_system, t[1]),
        )

    def _get_humidity(self, section):
        """
        Returns the humidity value
        """
        return self.__raw[section].get(WeatherField.HUMIDITY)

    def _get_visibility(self, section):
        """
        Returns the visibility value in current units
        """
        return MetricSystem.convert_distance(
            self.__report_metric_system, self.__metric_system,
            self.__raw[section].get(WeatherField.VISIBILITY))

    def _get_pressure(self, section):
        """
        Returns the barometric pressure value in current units
        """
        return MetricSystem.convert_pressure(
            self.__report_metric_system, self.__metric_system,
            self.__raw[section].get(WeatherField.PRESSURE))

    def _get_pressure_status(self, section):
        """
        Returns the barometric pressure status:
            steady (0), rising (1), or falling (2)
        """
        return self.__raw[section].get(WeatherField.PRESSURE_STATUS)

    def _get_barometer(self, section):
        """
        Returns the barometric pressure value in current units and its status
        in a tuple
        """
        return (self._get_pressure(section),
                self._get_pressure_status(section))

    def _get_wind_direction(self, section):
        """
        Returns the wind direction as string (N,E,S,W,NE,NW,SE,SW)
        """
        return self.__raw[section].get(WeatherField.WIND_DIRECTION)

    def _get_wind_speed(self, section, wind_unit = None):
        """
        Returns the wind speed in current units or in specified unit
        """
        if wind_unit is not None:
            return WindUnit.convert_speed(
                self.__report_wind_unit, wind_unit,
                self.__raw[section].get(WeatherField.WIND_SPEED))
        else:
            return WindUnit.convert_speed(
                self.__report_wind_unit, self.__wind_unit,
                self.__raw[section].get(WeatherField.WIND_SPEED))

    def _get_wind(self, section):
        """
        Returns a tuple containing wind direction and wind speed
        """
        wind_direction = self._get_wind_direction(section)
        wind_speed = self._get_wind_speed(section)
        if wind_direction is None or wind_speed is None:
            return None
        return (wind_direction, wind_speed)

    def _get_heat_index(self, section):
        """
        Returns the "feels like" temperature value in current units
        It uses the Heat Index
        """
        return MetricSystem.convert_temperature(MetricSystem.IMPERIAL,
            self.__metric_system, self._calc_heat_index_F(section))

    def _calc_heat_index_F(self, section):
        """
        Returns the "heat index" temperature value in Fahrenheit

        Author: Panagiotis Skintzos
        """
        # Source:
        # http://en.wikipedia.org/wiki/Heat_index
        # http://www.hpc.ncep.noaa.gov/html/heatindex.shtml
        hi = None
        # temperature (Fahrenheit)
        T = self._get_temperature(section, MetricSystem.IMPERIAL)
        # relative humidity (%)
        RH = self._get_humidity(section)
        if T is None or RH is None:
            return None
        if T <= 40.0:
            hi = T
        else:
            hitemp = 61.0 + ((T - 68.0) * 1.2) + ( RH * 0.094)
            hifinal = 0.5 * (T + hitemp)

            if hifinal > 79.0:
                c1 = -42.379
                c2 = 2.04901523
                c3 = 10.14333127
                c4 = -0.22475541
                c5 = -6.83783 * math.pow(10, -3)
                c6 = -5.481717 * math.pow(10, -2)
                c7 = 1.22874 * math.pow(10, -3)
                c8 = 8.5282 * math.pow(10, -4)
                c9 = -1.99 * math.pow(10, -6)
                T2 = math.pow(T, 2)
                RH2 = math.pow(RH, 2)

                hi = c1 + \
                    c2 * T + \
                    c3 * RH + \
                    c4 * T * RH + \
                    c5 * T2 + \
                    c6 * RH2 + \
                    c7 * T2 * RH + \
                    c8 * T * RH2 + \
                    c9 * T2 * RH2

                if (RH <= 13) and (T >= 80.0) and (T <= 112.0):
                    adj1 = (13.0 - RH) / 4.0
                    adj2 = math.sqrt((17.0 - math.fabs(T - 95.0)) / 17.0)
                    adj = adj1 * adj2
                    hi = hi - adj

                elif (RH > 85.0) and (T >= 80.0) and (T <= 87.0):
                    adj1 = (RH - 85.0) / 10.0;
                    adj2 = (87.0 - T) / 5.0;
                    adj = adj1 * adj2
                    hi = hi + adj
            else:
                hi = hifinal

        return hi

    def _get_dewpoint(self, section):
        """
        Returns the "dew point" temperature value in current units
        """
        return MetricSystem.convert_temperature(MetricSystem.SI,
            self.__metric_system, self._calc_dewpoint_C(section))

    def _calc_dewpoint_C(self, section):
        """
        Returns the "dew point" temperature value in Celsius
        Author: Panagiotis Skintzos
        """
        # Source:
        # http://en.wikipedia.org/wiki/Dew_point
        a = 17.271
        b = 237.7
        # temperature (Celsius)
        T = self._get_temperature(section, MetricSystem.SI)
        # relative humidity (%)
        RH = self._get_humidity(section)
        if T is None or RH is None:
            return None
        c = ((a * T)/(b + T)) + math.log(RH / 100.)
        dewpoint = (b * c) / (a - c)
        return dewpoint

    def _get_humidex(self, section):
        """
        Returns the humidex value
        Author: Felix Lawrence
        """
        # Source: http://en.wikipedia.org/wiki/Humidex
        # temperature (Celsius)
        T = self._get_temperature(section, MetricSystem.SI)
        # relative humidity (%)
        RH = self._get_humidity(section)
        # vapour pressure
        VP = self._calc_vapour_pressure(section) # (hPa, mbar)
        if T is None or RH is None or VP is None:
            return None
        return T + 0.5555 * (VP - 10.0) # (Celsius-like)

    def _get_apparent_temperature(self, section):
        """
        Returns the "feels like" value in current units
        It uses the Australian apparent temperature
        Author: Felix Lawrence
        """
        ## Australian Apparent Temperature
        return MetricSystem.convert_temperature(MetricSystem.SI,
            self.__metric_system, self._calc_apparent_temperature_C(section))

    def _calc_apparent_temperature_C(self, section):
        """
        Returns the "feels like" temperature value in Celsius
        Author: Felix Lawrence
        """
        # Australian Apparent Temperature
        # Source: http://www.bom.gov.au/info/thermal_stress/
        # temperature (Celsius)
        T = self._get_temperature(section, MetricSystem.SI)
        # relative humidity (%)
        RH = self._get_humidity(section)
        # vapour pressure
        VP = self._calc_vapour_pressure(section) # (hPa, mbar)
        # wind speed (metres/second)
        WS = self._get_wind_speed(section, WindUnit.MPS)
        if T is None or RH is None or VP is None or WS is None:
            return None
        #WS *= 0.44704 # metres/second
        # get feels like in Celsius
        return T + 0.33 * VP - 0.70 * WS - 4.00

    def _calc_vapour_pressure(self, section):
        """
        Returns the vapour pressure in hPa (=mbar) from temperature and humidity
        Author: Felix Lawrence
        """
        # Source: http://www.bom.gov.au/info/thermal_stress/
        # temperature (Celsius)
        T = self._get_temperature(section, MetricSystem.SI)
        # relative humidity (%)
        RH = self._get_humidity(section)
        if T is None or RH is None:
            return None
        return RH / 100. * 6.105 * math.exp(17.27 * T / (237.7 + T))

    def _format_time(self, value):
        """
        Formats a time value
        """
        return TimeFormatter.format_time(value)

    def _format_time_pair(self, value):
        """
        Formats a time pair
        """
        return "%s - %s" % (self._format_time(value[0]),
                            self._format_time(value[1]))

    def _format_timestamp(self, value):
        timedelta = datetime.datetime.now() - value
        days = timedelta.days
        hours = timedelta.seconds/3600
        if days > 0:
            return "%s %d %s:" % (__("day", "days", days), days, _("ago"))
        elif hours > 0:
            return "%s %d %s:" % (__("hour", "hours", hours), hours, _("ago"))
        return _("Now") + ":"

    def _format_day(self, value):
        """
        Formats a date object as day
        """
        t = datetime.datetime.today()
        if value.year == t.year and value.month == t.month and value.day == t.day:
            return _("Today") + ":"
        return value.strftime("%A:")

    def _format_temperature(self, value, precision = TEMPERATURE_PRECISION):
        """
        Formats a float temperature value
        """
        return "%s %s" % (NumberFormatter.format_float(value, precision),
            MetricSystem.get_temperature_unit(self.__metric_system, value))

    def _format_temperature_short(self, value):
        """
        Formats a float temperature value for minimal display
        """
        return "%s°" % (NumberFormatter.format_float(value, 0))

    def _format_temperature_pair(self, value, precision = 0):
        """
        Formats a tuple of float temperature values
        """
        return "%s - %s %s" % (
            NumberFormatter.format_float(value[0], precision),
            NumberFormatter.format_float(value[1], precision),
            MetricSystem.get_temperature_unit(self.__metric_system, value))

    def _format_humidex(self, value, precision = TEMPERATURE_PRECISION):
        """
        Formats a humidex value
        """
        return NumberFormatter.format_float(value, precision)

    def _format_humidity(self, value, precision = HUMIDITY_PRECISION):
        """
        Formats a humidity value
        """
        return "%s%%" % (NumberFormatter.format_float(value, precision))

    def _format_visibility(self, value, precision = DISTANCE_PRECISION):
        """
        Formats a visibility value with local units
        """
        return "%s %s" % (NumberFormatter.format_float(value, precision),
            MetricSystem.get_distance_unit(self.__metric_system, value))

    def _format_barometer(self, value, precision = PRESSURE_PRECISION):
        """
        Formats a barometric pressure value & status tuple with local units
        """
        return "%s %s %s" % (
            MetricSystem.convert_pressure_status_to_symbol(value[1]),
            NumberFormatter.format_float(value[0], precision),
            MetricSystem.get_pressure_unit(self.__metric_system, value[0]))

    def _format_wind(self, value, precision = WINDSPEED_PRECISION):
        """
        Formats a wind tuple (direction, speed) with local units
        """
        return "%s %s %s" % (
            WindUnit.convert_direction_to_symbol(value[0], value[1]),
            NumberFormatter.format_float(value[1], precision),
            WindUnit.get_speed_unit(self.__wind_unit, value[1]) )
